# Main.pm - The Object Class that provides a Main Object
#$Id: Base.pm,v 1.86.4.1 2006/08/01 23:09:38 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by James Pattie & Jason A. Pattie, 11/07/2000.

package Portal::Accounting::Base;
use strict;
use HTMLObject::Base;
use HTMLObject::Normal;
use HTMLObject::FrameSet;
use Portal::AppState;
use Portal::Data::Variables;
use Date::Calc qw(Today Add_Delta_Days);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::AppState Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.86.4.1 $';

# global variables.

=head1 NAME

Main - Object used to build a Main Menu for accounting package

=head1 SYNOPSIS

  use Portal::Accounting::Main;
  my $obj = Portal::Accounting::Main->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Main is the Main Menu from which every other part of
the systems will be accessed.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Main module.
 See Portal::AppState(3) for a listing of required arguments.

 Instantiates an instance of the Main State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( release => "", DBVersion => "", @_ );

  if ($self->error)
  {
    $self->error();
    return $self;
  }
  $self->{warningString} = "";
  $self->{warning} = 0;
  $self->{debugString} = "";
  $self->{debug} = 0;
  $self->{autoSubmit} = 1; #flag used to skip past status screens after saving data.

  # instantiate anything unique to this module
  my @state = split /::/, ref $self;
  for (my $i = 2; $i < @state; $i++)
  {
    if ($i != 2) { $self->{state} .= "::"; }
    $self->{state} .= $state[$i];
  }
  $self->{release} = $args{release};
  $self->{DBVersion} = $args{DBVersion};
  $self->{codeVersion} = $args{codeVersion};
  $self->{portalVariables} = Portal::Data::Variables->new(langObj => $self->{langObj});
  if ($self->{portalVariables}->error)
  {
    $self->error($self->{portalVariables}->errorMessage);
    return $self;
  }
  $self->{formObj} = HTMLObject::Form->new;
  if ($self->{formObj}->error)
  {
    $self->error($self->{formObj}->errorMessage);
    return $self;
  }
  $self->{jsDir}         = $self->{methods}->createBaseURL(type => "App",    linkType => "js",       appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{imageDir}      = $self->{methods}->createBaseURL(type => "App",    linkType => "image",    appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{cssDir}        = $self->{methods}->createBaseURL(type => "App",    linkType => "css",      appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{dbDump}        = $self->{methods}->createBaseURL(type => "App",    linkType => "db_dump",  appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{portalCGIDir}  = $self->{methods}->createBaseURL(type => "Portal", linkType => "cgibin",      appConfigObj => $self->{sessionObj}->{store}->{companyAppObj} );
  $self->{baseURL}       = $self->{methods}->createBaseURL(type => "App",    linkType => "cgi",      appConfigObj => $self->{sessionObj}->{store}->{companyAppObj});
  $self->{output} = { app => $self->{app}, state => $self->{state}, command => $self->{input}->{command} };

  #the state is not set until runtime
  $self->{postInfo} .= "<input type=\"hidden\" name=\"app\" value=\"$self->{app}\">\n";
  $self->{postInfo} .= "<input type=\"hidden\" name=\"state\" value=\"$self->{state}\">\n";
  my $windowName = '';
  if (defined $self->{input}->{windowName}) { $windowName = $self->{input}->{windowName}; }
  $self->{getInfo} .= "windowName=$windowName&app=$self->{app}&state=$self->{state}";

  $self->{OBJECT} = undef;  #instance of object just created/edited/saved/deleted
  # do validation
  if (!$self->Portal::Accounting::Base::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  return $self;
}

sub run
{
  my $self = shift;
  my $doc = $self->SUPER::run(@_);
  if (defined $doc && $doc->getContentType ne 'text/plain') { $doc->setFocus('body'); $doc->print($self->help); }
  return $doc;
}

sub prepDoc
{
  my $self = shift;
  my $message;
  if (defined $self->{input}->{MESSAGE}) { $message = $self->{input}->{MESSAGE}; }
  my %args = (doc => undef, @_);
  my %foo;
  my $doc;
  if (defined $args{doc}) { $doc = $args{doc}; }
  else { $doc = $self->SUPER::prepDoc(@_); }

  if (!defined $doc) {
    $doc = HTMLObject::Normal->new();
    $doc->setFocus('body');
    $doc->print("<div>Warining:  doc undefined from prepDoc.  creating new one.</div>")
  }
  my $colorScheme = $self->{applicationObj}->getCurrentColorScheme(userId => $self->{userObj}->{id});
  if ($self->{applicationObj}->error)
  {
    $self->error($self->{applicationObj}->errorMessage);
    return undef;
  }
  if (!defined $colorScheme)
  {
    $self->error("No valid colorScheme returned!<br>\n");
    return undef;
  }
  my %cssEntries = $self->{methods}->generateColorSchemeCSS(colorScheme => $colorScheme, appName => $self->{app});
  if ($self->{methods}->error)
  {
    $self->error($self->{methods}->errorMessage);
    return undef;
  }
  $doc->print(%cssEntries);

  my $title = $self->{langObj}->map($self->{commands}->{$self->{input}->{command}});
  $doc->setTitle( $title);
  $doc->setJavascriptInclude(file => $self->{jsDir}."Accounting.js");
  $doc->setJavascriptInclude(file => $self->{jsDir}."ujolibrary.js");
  if (defined $args{heading})
  {
    $foo{body} = $args{heading}.'<div id="error" class="error">'.$message."</div>\n";
  }
  else
  {
    $foo{body} = "<div class=\"title\">$title</div>\n".'<div id="error" class="error">'.$message."</div>\n";
  }
  $foo{javascript} = "langObj['rightClick'] = \"".$self->langObj->map("rightClick")."\";\n";
  $doc->print('  <link type="text/css" rel="styleSheet" href="'.$self->{cssDir}.'print.css" media="print">');
  $doc->print('  <link type="text/css" rel="alternate styleSheet" href="'.$self->{cssDir}.'print.css" title="Print" media="print">');
  $doc->print( %foo );
  $doc->print(css => [
    ".title { text-align: center; font-weight: bold; clear: both; margin-bottom: 1em; }",
    "body { font-size: 1em; }",
    "table,thead,tfoot,tbody,tr,td,th,input,form,select,multi-select { font-size: inherit; }",
    "form { padding: 0; margin: 0; }",
  ] );
  #add colors to xiwa here ingoring portal color schemes until james and I work out something.
  $doc->print(css => [
    ".primary { color: #000000; background-color: #99cccc; padding: .7em; margin: 1em; }",
    ".secondary { color: #000000; background-color: #669999; padding: .5em; margin: .5em; }",
    ".tertiary { color: #FFFFFF; background-color: #669999; }",
    ".menu { color: #000000; background-color: #99cccc; }",
    ".controls { font-size: small; }",
    ".debug { display: none; }",
    ".help { padding: .8em; margin: .5em; }",
    ".pager { text-align: right; }",
    ".negative { color: #ff0000; }",
    ".printInline { display: none; }",
    ".printBlock { display: none; }",
    '.deleting { color: #cccccc; } ',
  ] );
  return $doc;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->error();
    return 0;
  }

  # validate our parameters.
  if (length $self->{release} == 0)
  {
    $self->invalid("release", $self->{release});
  }
  if (length $self->{DBVersion} == 0)
  {
    $self->invalid("DBVersion", $self->{DBVersion});
  }
  if (length $self->{codeVersion} == 0)
  {
    $self->invalid("codeVersion", $self->{codeVersion});
  }
  if (not defined $self->{methods})
  { $self->missing("methods");  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub mainButton
{
  my $self = shift;
  my $return;
  my $display = $self->{langObj}->map($self->{commands}->{display});
  $return = << "ENDOF";
  <form name="display">
    <input type="submit" value="$display">
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="command" value="display">
  </form>
ENDOF
  return $return;
}

sub c_new
{
  my $self = shift;
  return $self->c_edit(@_);
}

sub c_save
{
  my $self = shift;
  return $self->c_edit(@_);
}

=item HTMLObject c_edit(arg1 =>, arg2 =>, . . .)

  this routine is to create/edit a new object.  It will present the
  appropriate screen for the correct action.
  arguments such as ledgerCode can be sent and will be passed to objects

=cut
sub c_edit
{
  my $self = shift;
  my $doc = $self->prepDoc;
  $doc->setFocus('body');
  my %args = ( command => $self->{input}->{command}, @_ );
  my $command = $args{command};
  my $l = $self->{langObj};
  my $foo = $self->{OBJECT};
  if ($command eq "save")
  { #validate user input then react
    my $v = $foo->validateForm(input => $self->{input});
    if (!$v) { $self->error("<p>ValidateForm failed!</p>\n".$foo->errorMessage); }
#     my $p = $foo->populate(%{$self->{input}});
#     if ($v && $p)
    elsif ($foo->populate(%{$self->{input}}))
    {
      my $r = $foo->write(); #insert/update into database
      $doc->print($foo->debugString);
      if (!$foo->error)
      {
        $doc->print($l->map("Success"));  #!!! let this fall through to child when portal is fixed
        $doc->print($foo->debugString);
        if ($self->{autoSubmit} && (!defined $self->{input}->{embed} || $self->{input}->{embed} == 0 ))
        {
          $doc->setOnload('return document.display.submit();');
        }
        return $doc;
      }
      else
      {
        $doc->print('FAILED'. $foo->errorMessage);
      }
    }
#     else
#     {
#       $doc->print("<p>$v $p</p>");
#     }
  }
  elsif ($command eq "edit")
  {
    if (! defined $self->{input}->{$foo->{id}}) { $doc->print("<p>To use this object, there must be an id field of $foo->{id} sent in the data!</p>\n");  return $doc; }
    my $result = $foo->set($foo->{id}, $self->{input}->{$foo->{id}});
    if (!$foo->error) { $foo->select(); }
    $doc->print($foo->debugString);
    $doc->print($foo->errorString);
  }
  #display form
  if ($foo->error)
  {
    $self->error($foo->errorMessage);
  }
  $foo->resetError;
  my %form = $foo->printForm();
  if (defined %form)
  {
    $doc->print(%form);
  }
  else
  {
    $doc->print('error! form is undefined<br>');
    $doc->print("\nformObj:".$foo->{formObj}->error);
  }
  if ($self->error) { $doc->print($self->errorMessage); }
  $doc->print($foo->debugString);
  if (defined $self->{input}->{embed} && $self->{input}->{embed} == 1)
  {
    $doc->onload('window.parent.hide("pleaseDiv");');
  }
  return $doc;
}

=item HTMLObject c_delete()

  presents screens to delete the working object.

=cut
sub c_delete
{
  my $self = shift;
  my %args = (doc => undef, @_);
  my $l = $self->{langObj};
  my $Failed = $l->map("Failed");
  my $Success = $l->map("Success");
  my $confirm = $l->map("confirmDelete");
  my $doc;
  if (defined $args{doc}) { $doc = $args{doc}; }
  else { $doc = $self->prepDoc();  }

  my $foo = $self->{OBJECT};
  $foo->populate(%{$self->{input}});
  if ($foo->error)
  {
    $doc->print("Populate failed! $foo->{id}:$self->{input}->{id}<br>\n".$foo->errorMessage."<br>".$foo->debugString);
    return $doc;
  }
  $foo->select();
  if ($foo->error)
  {
    $doc->print("<p>SELECT for delete failed!<br>\n".$foo->errorMessage."<br>".$foo->debugString."FOO</p>");
    return $doc;
  }
  if (defined $self->{input}->{confirmDelete} && $self->{input}->{confirmDelete} eq "1")
  {#do the actual delete
    my $result = $foo->delete();
    if ($foo->error)
    {
      $doc->print($foo->errorMessage);
    }
    else
    {
      my $out = $l->map("Delete") . ": ";
      if ($result == -1)
      {
        $out .= "$Failed<br>".$foo->result;  #is this right?
      }
      elsif ($result >= 0)  {  $out .= "$Success<br>";  }
      else { $out .= "<p>Error in result: '$result'</p>\n";  }
      $doc->print($out."<hr>". $foo->debugString);
    }
    return $doc;
  }
  else
  { #output confirm d
    my $led = "";
    if (defined $args{ledgerCode}) { $led = " <input type=\"hidden\" name=\"ledgerCode\" value=\"$args{ledgerCode}\">\n";  }
    my $yes = $self->{langObj}->map("Yes");
    $doc->print(<<"ENDOF");
<form name="mainForm">
  <p>$confirm<br>
  <input type="submit" name="yes" value="$yes"></p>
  <input type="hidden" name="app" value="$self->{app}">
  <input type="hidden" name="state" value="$self->{state}">
  <input type="hidden" name="command" value="delete">
  <input type="hidden" name="confirmDelete" value="1">
  <input type="hidden" name="id" value="$self->{input}->{id}">
  $led
</form>
ENDOF
  }
  return $doc;
}

sub getDateArray
{
  my $self = shift;
  return Today();
}

sub c_display
{
  my $self = shift;
  my $doc = $self->prepDoc(@_);
  my $confirm = $self->{langObj}->map("Confirm Delete");
  my $foo = $self->{OBJECT};
  $doc->print($foo->printList);
  $doc->print($foo->printNew);
  $doc->print($foo->debugString);
  $doc->setFocus("javascript");
  $doc->print(<<"ENDOF");
function deleteConfirm(myForm)
{
  if (confirm("$confirm"))
  {
    myForm.confirmDelete.value = 1;
    return true;
  }
  return false;
}
ENDOF
  return $doc;
}

=item boolean warning("string") or boolean warning()

  similar to the error routine this routine will set a warning
  statement and turn on warning if called.  access throught
  $obj->warningString

=cut
sub warning
{
  my $self = shift;
  my @callerArgs = caller(1);
  (my $subName = $callerArgs[3]) =~ s/^(.+)(::)([^:]+)$/$1->$3/;
  my $callerErrStr = "";

  if (scalar @_ > 0)
  {
    # we are setting an warning condition.
    if (scalar @_ == 1)
    {
      $self->{warningString} .= $callerErrStr . @_[0]."<br>";
    }
    else
    {
      # handle the sprintf case.
      if (@_[0] =~ /(?<!\\)%[^%]/)
      {
        # build up the string to eval for the sprintf.
        my $str = "\"@_[0]\"";
        for (my $i=1; $i < scalar @_; $i++)
        {
          $str .= ", \"@_[$i]\"";
        }
        $self->{warningString} .= $callerErrStr;
        eval "\$self->{warningString} .= sprintf($str);";
        if ($@)
        {
          $self->warning($callerErrStr . $@);
          return;
        }
      }
      else
      {
        $self->{warningString} .= $callerErrStr . join(" ", @_);
      }
    }
    $self->{warningString} .= "\n";
    $self->{warning} = 1;
    $self->log("WARN ".$self->{warningString});
  }

  return $self->{warning};
}

=item boolean debug("string") or boolean debug()

  similar to the error routine this routine will set a debug
  statement and turn on debug if called.  access throught
  $obj->debugString

=cut
sub debug
{
  my $self = shift;
  my @callerArgs = caller(1);
  (my $subName = $callerArgs[3]) =~ s/^(.+)(::)([^:]+)$/$1->$3/;
  my $callerErrStr = "$subName";

  if (scalar @_ > 0)
  {
    # we are setting an error condition.
    if (scalar @_ == 1)
    {
      $self->{debugString} .= $callerErrStr . @_[0]."<br>";
    }
    else
    {
      if (@_[0] eq "debugString")
      {
        my %args = ( @_ );
        if (!exists $args{debugString})  # make sure we get the debugString argument!
        {
          $self->debug($callerErrStr . "<b>debugString</b> is missing!<br>\n");
          return;
        }
        else
        {
          $self->{debugString} .= $callerErrStr . $args{debugString};
        }
      }
      else
      {
        # handle the sprintf case.
        if (@_[0] =~ /(?<!\\)%[^%]/)
        {
          # build up the string to eval for the sprintf.
          my $str = "\"@_[0]\"";
          for (my $i=1; $i < scalar @_; $i++)
          {
            $str .= ", \"@_[$i]\"";
          }
          $self->{debugString} .= $callerErrStr;
          eval "\$self->{debugString} .= sprintf($str);";
          if ($@)
          {
            $self->debug($callerErrStr . $@);
            return;
          }
        }
        else
        {
          $self->{debugString} .= $callerErrStr . join(" ", @_);
        }
      }
    }
    $self->{debugString} .= "\n";
    $self->{debug} = 1;
    $self->log("DEBUG ".$self->{debugString});
  }

  return $self->{debug};
}

=item negativeStyle

 requires: value
 optional: format
 returns: formatted html
 summary: This routine wraps the value given in the desired
   css format based on it being negative or positive.
   if the value is not a number then it is simply returned unmodified.
 format options:
   0 => <span class="negative">(amount)</span> | <span class="positive">amount</span>
   1 => <span class="negative">-amount<span>   | <span class="positive">amount</span>
   2 => <span class="negative">amount-</span>  | <span class="positive">amount</span>

=cut
sub negativeStyle
{
  my $self = shift;
  my %args = (format => 0, value => '', @_);
  if ($args{value} =~ /-/)
  {
    if ($args{value} >= 0) { return "<span class=\"positive\">$args{value}</span>"; }
    if ($args{format} == 0) { return "<span class=\"negative\">($args{value})</span>"; }
    elsif ($args{format} == 1) { return "<span class=\"negative\">-$args{value}</span>"; }
    elsif ($args{format} == 2) { return "<span class=\"negative\">$args{value}-</span>"; }
    else { $self->warning("<p>Invalid format option given to negativeStyle: $args{format}</p>\n"); }
  }
  return $args{value};
}

=item commify

 requires: value
 optional: US => 0|1 - defaults to 1
 returns: value with thousands seperators
 This routine commifies the specified value according to standards.
 An alternative of EU standards would not be hard.

 !!!CURRENTLY BROKEN WHY TRYING TO DO NON US version

=cut
sub commify
{
  my $self = shift;
  my $seperator = ',';
  my $decimal = '\.';
  my $value;
  if (scalar @_ > 1)
  {
    my %args = ( value => "0", US => 1, @_ );
    if (!$args{US})
    {
      $seperator = '.';
      $decimal = ',';
    }
    $value = reverse $args{value};
  }
  else
  {
    $value = reverse @_[0];
  }
#  $value =~ s/(\d\d\d)(?=\d)(?!\d*\.)/$1,/g;
#  $value =~ s/(\d\d\d)(?=\d)(?!\d*\,)/$1./g;
  eval('$value =~ s/(\d\d\d)(?=\d)(?!\d*'.$decimal.')/$1'.$seperator.'/g;');
#  print '$value =~ s/(\d\d\d)(?=\d)(?!\d*'.$decimal.')/$1'.$seperator.'/g;'."\n";

  $value = reverse $value;
  return $value;
}

sub c_searchForm
{
  my $self = shift;
  my $doc;
  my %args = ( command => $self->{input}->{command}, doc => undef, @_ );
  if (!defined $args{doc}) { $doc = $self->prepDoc; }
  my $command = $args{command};
  my $l = $self->{langObj};
  my $foo = $self->{OBJECT};
  if ($foo->error) { $doc->print($foo->errorMessage); return $doc;  }
  if ($command eq "search")
  { #validate user input then react
    if ($foo->populate(%{$self->{input}}) && $foo->validateForm(input => $self->{input}, formName => "main"))
    {
      my @je = $foo->selectArray();
      if ($foo->error) { $doc->print($foo->errorMessage); }
      else
      {
       # then display them by walking and $o->printHTML
        if (scalar @je == 0) { $doc->print("<p>".$l->map("Found")." 0</p>\n"); }
        else
        {
          #output search paging form if it exists
          if (defined $foo->{data}->{pager})
          {
            my %form = $foo->pagerForm(number => $self->{input}->{number});
            if ($foo->error) { $doc->print($foo->errorMessage); }
            else { $doc->print(%form); }
          }
          $doc->print($je[0]->printHeaders);
          foreach my $bar (@je)
          {
            $bar->isValid;
            $doc->print($bar->printForm);
#            $doc->print($bar->printXML);
#            $doc->print($bar->printHTML);
#!!! use htmlobject error console here
            $doc->print($bar->errorMessage);
            $doc->print($bar->debugString);
          }
          if ($self->warning) { $doc->print($self->warningString); }
          $doc->print($self->debugString);
          return $doc;
        }
      }
    }
  }
  #display form
  my %form = $foo->printForm();
  if ($foo->error)
  {
    $self->error($foo->errorMessage);
  }
  if (defined %form)
  {
    $doc->print(%form);
  }
  else
  {
    $doc->print('error! form is undefined<br>');
    $doc->print("\nformObj:".$foo->{formObj}->error);
  }

  $doc->print($foo->debugString);
  return $doc;
}

sub c_search
{
  my $self = shift;
  return $self->c_searchForm(@_);
}

sub demoCheck
{
  my $self = shift;
#  if (!$self->{userObj}->{admin}) {  return 0;  }
  my $sql = "SELECT value FROM config_tb WHERE name = 'demo'";
  my $sth = $self->dbHandle->read($sql);
  if (my @info = $sth->fetchrow_array) { return $info[0]; }
  return 0;
}

=item string JER(string)

  JER stands for JSON error or Javascript Error.  This routine will take the given error message and encode it for use with the javascript ERROR() function and return the string to output into the text/plain document

=cut
sub JER
{
  my $self = shift;
  my $m = shift;
  $m =~ s/'/\\'/g;
  $m =~ s/\n//g;
  return "ERROR('$m');\n";
}

sub c_lock
{
  my $self = shift;
  my $doc = $self->prepDoc;
  my $message;
  my $r = $self->{applicationObj}->lockApp(app => $self->{appObj}->{id}, type => "c", company => $self->{companyObj}->{id});
  if ($self->{appConfigObj}->error) { $message = '<div class="error">There was an error while locking the app</div>'.$self->{appConfigObj}->errorMessage; }
  else { $message = "<div>Application $self->{app} is now locked by you.  Non Admin users cannot access the application until you unlock it.</div>";}
  $doc->print(<<"ENDOF");
  $message
  <form target="_top">
    <input type="hidden" name="app" value="$self->{app}" />
    <input type="hidden" name="state" value="Main" />
    <input type="hidden" name="command" value="display" />
    <input type="submit" name="Submit" value="Main Menu" />
  </form>
ENDOF

  return $doc;
}

sub c_unlock
{
  my $self = shift;
  my $doc = $self->prepDoc;
  my $message;
  my $r = $self->{applicationObj}->unlockApp(app => $self->{appObj}->{id}, type => "c", company => $self->{companyObj}->{id});
  if ($self->{appConfigObj}->error) { $message = '<div class="error">There was an error while locking the app</div>'.$self->{appConfigObj}->errorMessage; }
  else { $message = "<div>Application $self->{app} is now unlocked.  All assigned users can access the application.</div>";}
  $doc->print(<<"ENDOF");
  $message
  <form target="_top">
    <input type="hidden" name="app" value="$self->{app}" />
    <input type="hidden" name="state" value="Main" />
    <input type="hidden" name="command" value="display" />
    <input type="submit" name="Submit" value="Main Menu" />
  </form>
ENDOF
  return $doc;
}

sub help
{
  my $self = shift;
  my $foo = 'help_'.$self->{input}->{state}.'_'.$self->{input}->{command};
  my $help = '';
  if (defined $self->{langObj}->{phraseObj}->{phrases}->{$foo})
  {
    $help = "\n".'<div style="clear:both;" class="tertiary help">'."\n".$self->{langObj}->map($foo)."\n</div>\n";
  }
#  else { $help = "No help for '$foo'" ; }
  return $help;
}

sub accountMap
{
  my $self = shift;
  my $r;
  my $sql = "SELECT first_digit AS meta, name, sub_acct_type FROM acct_type_tb ORDER BY first_digit, name";
  my $subs = "SELECT name, num AS id FROM sub_acct_type_tb";
  my %subs = ();
  my $sth = $self->dbHandle->read(sql => $subs);
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); return undef; }
  while (my $d = $sth->fetchrow_hashref)
  {
    $subs{$d->{id}} = $d->{name};
  }

  my $sth = $self->dbHandle->read(sql => $sql);
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); return undef; }
  $r .= '<div id="showAccountMap" style="margin: 1em; padding: 1em;" onclick="document.getElementById(\'accountMap\').style.display = \'table\'; this.style.display = \'none\';">Click Here to see a chart of links between Account Types and Sub Account Types.  This will affect the way data is entered in data entry sceens such as Check and Deposit.</div>';
  $r .= '<table style="display: none;" id="accountMap">';
  my $X = '<span onclick="document.getElementById(\'showAccountMap\').style.display = \'block\'; document.getElementById(\'accountMap\').style.display = \'none\';" title="Close" style="color: red; ">X</span>';
  $r .= "\n<thead><tr><th>Meta</th><th>Account Type</th><th>Sub Account Types</th><th>$X</th></tr></thead>\n<tbody>\n";
  #parse sub_types from each record and replace with names of sub types
  while (my $d = $sth->fetchrow_hashref)
  {
    my @types = split(',',$d->{sub_acct_type});
    for (my $t = 0; $t < scalar @types; $t++)
    {
      $types[$t] = $subs{$types[$t]};
    }
    my $types = join(',',@types);
    $r .= <<"ENDOF";
  <tr>
    <td>
      $d->{meta}
    </td>
    <td>
      $d->{name}
    </td>
    <td>
      $types
    </td>
  </tr>
ENDOF
  }
  $r .= "</tbody></table>\n";
  return $r;
}

sub log
{
  my $self = shift;
  my $text = shift;
  my $sql = "INSERT INTO log_tb (text) VALUES (?)";
  $self->{dbHandle}->write(sql => $sql, plug => [$text]);
  if ($self->{dbHandle}->error)
  {
    $self->error($self->dbHandle->errorMessage);
    return 0;
  }
  return 1;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
