# Main.pm - The Object Class that provides a Main Object
#$Id: BaseL.pm,v 1.29 2006/05/04 20:25:42 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by James Pattie & Jason A. Pattie, 11/07/2000.

package Portal::Accounting::BaseL;
use strict;
use Portal::Accounting::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);
require Exporter;

@ISA = qw(Portal::Accounting::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.29 $';

# global variables.

=head1 NAME

BaseL - Object used as parent to derive Ledger specific states.

=head1 SYNOPSIS

  use Portal::Accounting::Main;
  my $obj = Portal::Accounting::Main->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Main is the Main Menu from which every other part of
the systems will be accessed.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Main module.
 See Portal::AppState(3) for a listing of required arguments.

 Instantiates an instance of the Main State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # do validation
  if (!$self->Portal::Accounting::BaseL::isValid)
  {
    return $self;
  }
  $self->{postInfo} .= "<input type=\"hidden\" name=\"ledgerCode\" value=\"$self->{input}->{ledgerCode}\">\n";
  $self->{getInfo} .= "&ledgerCode=".$self->{input}->{ledgerCode};

  return $self;
}

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }
  # do validation code here.
  if (not defined $self->{input}->{ledgerCode})
  { $self->missing("ledgerCode");  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }
  return 1;
}

sub prepDoc
{
  my $self = shift;
  my $message;
  if (defined $self->{input}->{MESSAGE}) { $message = $self->{input}->{MESSAGE}; }
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $color = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{color};
  my $title = $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{name}." ". $self->{langObj}->map($self->{commands}->{$self->{input}->{command}});
  my $heading = '<div class="title"><span class="ledgerFGColor">'. $self->{sessionObj}->{store}->{ledgers}->{$ledgerCode}->{name}."</span> ". $self->{langObj}->map($self->{commands}->{$self->{input}->{command}})."</div>";
  my %args = (doc => undef,
    title => $title,
    heading => $heading,
    command => $self->{input}->{command},
  @_);
  my %foo;
  my $doc;
  if (defined $args{doc}) { $doc = $args{doc}; }
  else {
    $doc = HTMLObject::Normal->new;
    $doc->setTitle($args{title});
    $doc->print($args{heading});
  }
  $doc->print(body => '<div id="error" class="error">'.$message.'</div>');
  $doc->setJavascriptInclude(file => $self->{jsDir}."Accounting.js");
  $doc->setJavascriptInclude(file => $self->{jsDir}."ujolibrary.js");

  my $colorScheme = $self->{applicationObj}->getCurrentColorScheme(userId => $self->{userObj}->{id});
  if ($self->{applicationObj}->error)
  {
    $self->prefixError($self->{applicationObj}->errorMessage);
    return undef;
  }
  if (!defined $colorScheme)
  {
    $self->error("No valid colorScheme returned!<br>\n");
    return undef;
  }
  my %cssEntries = $self->{methods}->generateColorSchemeCSS(colorScheme => $colorScheme, appName => $self->{app});
  if ($self->{methods}->error)
  {
    $self->error($self->{methods}->errorMessage);
    return undef;
  }
  $doc->print(%cssEntries);
  #add css style ledgerBGColor, ledgerFGColor to each page individually bc they change for each ledger
  $doc->print(css => [ ".ledgerFGColor { color: $color; }", ".ledgerBGColor { background-color: $color; }", ".ledgerBorderColor { border: none; border-style: solid ; border-color: $color; }" ]);
  $doc->print(css => [
    ".title { text-align: center; font-weight: bold; clear: both; margin-bottom: 1em; }",
    "body { font-size: 1em; }",
    "table,thead,tfoot,tbody,tr,td,th,input,form,select,multi-select { font-size: inherit; }",
    "form { padding: 0; margin: 0; }",
  ] );
  #add colors to xiwa here ingoring portal color schemes until james and I work out something.
  $doc->print(css => [
    ".primary { color: #000000; background-color: #99cccc; padding: .7em; }",
    ".secondary { color: #000000; background-color: #669999; padding: .5em; }",
    ".tertiary { color: #FFFFFF; background-color: #669999; }",
    ".menu { color: #000000; background-color: #99cccc; }",
    ".controls { font-size: small; }",
    ".debug { display: none; }",
    ".help { padding: .8em; margin: .5em; }",
    ".pager { text-align: right; }",
    ".negative { color: #ff0000; }",
    ".printInline { display: none; }",
    ".printBlock { display: none; }",
    '.deleting { color: #cccccc; } ',
  ] );

  $doc->setFocus('head');
  $doc->setLink(type => 'text/css',  rel => "styleSheet", href => $self->{cssDir}.'print.css', media => "print");
  $doc->setLink(type => "text/css", rel => "alternate styleSheet", href => $self->{cssDir}."print.css", title => "Print", media => "print");
#   $doc->print('  <link type="text/css" rel="styleSheet" href="'.$self->{cssDir}.'print.css" media="print">');
#   $doc->print('  <link type="text/css" rel="alternate styleSheet" href="'.$self->{cssDir}.'print.css" title="Print" media="print">');

  $doc->setFocus('body');
  return $doc;
}

sub mainButton
{
  my $self = shift;
  my $return;
  my $display = $self->{langObj}->map($self->{commands}->{display});
  $return = << "ENDOF";
  <form name="display">
    <input type="submit" value="$display">
    <input type="hidden" name="app" value="$self->{app}">
    <input type="hidden" name="state" value="$self->{state}">
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}">
    <input type="hidden" name="command" value="display">
  </form>
ENDOF
  return $return;
}

sub c_new
{
  my $self = shift;
  return $self->SUPER::c_new(ledgerCode => $self->{input}->{ledgerCode}, @_);
}

sub c_save
{
  my $self = shift;
  return $self->SUPER::c_save(ledgerCode => $self->{input}->{ledgerCode}, @_);
}

sub c_edit
{
  my $self = shift;
  return $self->SUPER::c_edit(ledgerCode => $self->{input}->{ledgerCode}, @_);
}

=item HTMLObject c_delete()

  presents screens to delete the working object.

=cut
sub c_delete
{
  my $self = shift;
  return $self->SUPER::c_delete(ledgerCode => $self->{input}->{ledgerCode}, @_);
}

sub c_display
{
  my $self = shift;
  my $doc = $self->prepDoc(@_);
  my $confirm = $self->{langObj}->map("Confirm Delete");
  my $foo = $self->{OBJECT};
  $doc->print($foo->printList);
  $doc->print($foo->printNew);
  $doc->print($self->mainButton);
  if ($foo->error)
  {
    $doc->print($foo->errorMessage);
  }
  $doc->print($foo->debugString);
  $doc->setFocus("javascript");
  $doc->print(<<"ENDOF");
function deleteConfirm(myForm)
{
  if (confirm("$confirm"))
  {
    myForm.confirmDelete.value = 1;
    return true;
  }
  return false;
}
ENDOF
  return $doc;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
