# DB::Upgrade.pm - The Object Class that provides a DB::Upgrade Object
#$Id: Upgrade.pm,v 1.14.4.1 2007/02/19 00:05:49 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::DB::Upgrade;
use strict;
use Portal::Accounting::Main;
use Portal::Accounting::DB::Main;
use File::Basename qw( fileparse );
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Main Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.14.4.1 $';

# modify to represent the commands this state provides.
my %commands = (
  display => "Initialize the Upgrade Process",
  start => "Upgrade",
  backup => "Backup",
  pre => "Collect Data",
  upgrade => "Upgrade",
  post => "Post Upgrade",
  finish => 'Upgrade Finished',
  topFrame => 'Upgrade Database',
);
#each step will validate the data from the step before and send it back if something went wrong.

=head1 NAME

DB::Upgrade

=head1 SYNOPSIS

  use Portal::Accounting::DB::Upgrade;
  my $obj = Portal::Accounting::DB::Upgrade->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

DB::Upgrade allows for migration from one version of the database to the
next and allows the user to interact with the migration in a nicer
environment than a command line provides.

This is run from the Accounting module when it detects that a new
version of the database is available (code wise) and the database is not
at the version needed.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new()

 Creates a new instance of the Portal::Accounting::DB::Upgrade module.
 See Portal::AppState(3) for a listing of required arguments.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->error('');
  }

  # instantiate anything unique to this module
  $self->{upgradePath} .= "/$VERSION";
  $self->{upgradeFile} = $self->{upgradePath}."/upgrade.psql";

  # do validation
  if (!$self->Portal::Accounting::DB::Upgrade::isValid)
  {
    # the error is set in the isValid() method.
  }

  # do anything else you might need to do.
  $self->{dbHandle} = $args{dbHandle};
  $self->{langObj} = $args{langObj};
  $self->{upgradePath} = $self->{configObj}->{webRoot} . $self->{configObj}->{siteDir} . "/install/Accounting/db/";
  $self->{commands} = \%commands;

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut
sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject run(command)

    Runs the specified command and returns the resulting HTMLObject.

=cut
sub run
{
  my $self = shift;
  my %args = (runAround => 0, @_);
  if ($args{runAround}) { return $self->SUPER::run(@_); }

  my $doc = $self->prepDoc;
  use Portal::Accounting::DB::Main;
  my $db = Portal::Accounting::DB::Main->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($db->error) { $doc->print($db->errorMessage); return $doc; }
  my $Vdb = $db->getDBVersion;
  if ($db->error) { $doc->print($db->errorMessage); return $doc; }
  if (!$self->{userObj}->{admin})
  {
    $doc->print("You must be an admin to upgrade the database.");
    return $doc;
  }
  if ($Vdb eq $self->{DBVersion})
  {
    $doc->print("Database does not need to be upgraded.");
    return $doc;
  }
  my $version = $self->release ."_".$self->{DBVersion};
  my $module;
  eval("use Portal::Accounting::DB::V".$version.";");
  eval('$module = Portal::Accounting::DB::V'.$version.'->new(codeVersion => $self->{codeVersion}, DBVersion => $self->{DBVersion}, release => $self->{release}, langObj => $self->{langObj}, dbHandle => $self->{dbHandle},'.$self->arguments.');');
  if (! defined $module)
  {
    $doc->print("<div>Error while instantiating module.  module is undefinded.<br>");
    $doc->print("use Portal::Accounting::DB::V".$version.";<br>");
    $doc->print('$module = Portal::Accounting::DB::V'.$version.'->new(codeVersion => $self->{codeVersion}, DBVersion => $self->{DBVersion}, release => $self->{release}, langObj => $self->{langObj}, dbHandle => $self->{dbHandle},'.$self->arguments.');');
    $doc->print("</div>");
    return $doc;
  }
  if ($module->error) { $doc->print($module->errorMessage); return $doc; }
  my $doc2 = $module->run(@_, runAround => 1);
#  eval('$doc = $module->c_'.$self->{input}->{command}.';');
  $doc->setFocus('body');
  if (! defined $doc2)  { $doc->print("<div>Error!  V$version module returned undefinded doc!</div>"); }
  else { $doc = $doc2; }
  if ($module->error)
  {
    $doc->print($module->errorMessage);
    $doc->print(<<"ENDOF");
<form name="main">
  There was an error.  You may restore the old database by clicking below.
  <input type="hidden" name="app" value="$self->{app}" />
  <input type="hidden" name="state" value="DB::Upgrade" />
  <input type="hidden" name="command" value="restore" />
  <input type="Submit" name="Submit" value="Restore" />
</form>
ENDOF
  }
  return $doc;
}

sub c_display
{
  my $self = shift;
  use HTMLObject::FrameSet;
  my $doc = HTMLObject::FrameSet->new;
  $doc->setTitle("Database Upgrade to $self->{dbVersion}");
  my $innerContent;
  $innerContent = $doc->createNoframe("You must have a frames capable browser!");
  my $db = $doc->encodeString('DB::Upgrade');
  $innerContent .= $doc->createFrame(src => "?app=Accounting&state=$db&command=topFrame", frameborder => 'no', scrolling => 'auto', border => '1', name => 'upgradeTop');
  $innerContent .= $doc->createFrame(src => "?app=Accounting&state=$db&command=start", frameborder => 'no', scrolling => 'auto', name => 'upgradeBottom');
  $doc->outerFrameset(rows => '20%, *', content => $innerContent, border => '1', frameborder => 'no');
  return $doc;
}

sub c_topFrame
{
  my $self = shift;
  my $doc = $self->prepDoc;
  $doc->setFocus('body');
  $doc->print("<center>Need to Upgrade database to <b>$self->{release}.$self->{DBVersion}</b></center>");
  $doc->print('<div id="console"></div>');
  $doc->setFocus('javascript');
  $doc->print(<<"ENDOF");
function foo(zoo)
{
  CONSOLE(zoo);
}
ENDOF
  return $doc;
}

sub c_start
{
  my $self = shift;
  my %args = (command => 'backup', doc => $self->prepDoc, @_);
  my $doc = $args{doc};

  if (!$self->error) {
    $doc->print(<<"ENDOF");
  Click to start the upgrade.
  <form name="main">
    <input type="hidden" name="app" value="$self->{app}" />
    <input type="hidden" name="state" value="DB::Upgrade" />
    <input type="hidden" name="command" value="$args{command}" />
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}" />
    <input type="submit" name="Submit" value="Begin" onclick="parent.upgradeTop.foo('Please Wait . . . '); this.disabled = true;"/>
  </form>
ENDOF
  }
  return $doc;
}

sub c_backup
{
  my $self = shift;
  my %args = (command => 'pre', doc => $self->prepDoc, @_);
  my $doc = $args{doc};
  my $confirm = $self->{input}->{confirm};
  my $db = Portal::Accounting::DB::Main->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($db->error) { $doc->print($db->errorMessage); return $doc; }
  my $dbVersion = $db->getDBVersion;
  my $dumpDir = $self->{configObj}->{webRoot} . $self->{configObj}->{siteDir} . "/db_dump/$self->{app}/$self->{companyObj}->{code}/";
  my $file = "xiwa_$dbVersion.dump";

  if (defined $confirm && $confirm eq 'YES')
  {
    # dump the database
    my $companyAppObj = $self->{sessionObj}->{store}->{companyAppObj};
    my $result = $self->{methods}->dbDump(
      type => $companyAppObj->{dbType},
      host => $companyAppObj->{dbHost},
      port => $companyAppObj->{dbPort},
      user => $self->{configObj}->{dbUser},
      passwd => $self->{configObj}->{dbPasswd},
      dbName => $companyAppObj->{dbName},
      dir => $dumpDir,
      fname => $file,
      inserts => 1,
    );
    if ($self->{methods}->error)
    {
      $self->error($self->{methods}->errorMessage);
    }
    else
    {
      $doc->print("Backed up database version $dbVersion to file $file.");
    }
    if (!$self->error)
    {
      $doc->print(<<"ENDOF");
<form>
  <input type="hidden" name="app" value="$self->{app}" />
  <input type="hidden" name="state" value="DB::Upgrade" />
  <input type="hidden" name="command" value="$args{command}" />
  <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}" />
  <input type="submit" name="Submit" value="Collect Data" onclick="parent.upgradeTop.foo('Collecting data from user'); this.disabled = true;"/>
</form>
ENDOF
    }
  }
  elsif (defined $confirm && $confirm eq 'NO')
  {
    $doc->print(<<"ENDOF");
  <form>
    <input type="hidden" name="app" value="$self->{app}" />
    <input type="hidden" name="state" value="Main" />
    <input type="hidden" name="command" value="$args{command}" />
    <input type="submit" name="Submit" value="Collect Data" onclick="this.disabled = true;" />
  </form>
ENDOF
  }
  else
  {
    my $warning = '';
    my $yes = ' checked="1"';
    my $no = '';
    #check to see if a backup file already exists and warn user.
    if (-e "$dumpDir/$file")
    {
      $warning = "This will overwrite a pre-existing file!";
      $no = 'checked = "1"';
      $yes = '';
    }
    #set yes/no based on whether file exists
    $doc->print(<<"ENDOF");
<form name="main">
  <p>Do you want to backup this database version '$dbVersion' to
    '${dumpDir}$file' ?<br>
    $warning
  </p>
  <input type="hidden" name="app" value="$self->{app}" />
  <input type="hidden" name="state" value="DB::Upgrade" />
  <input type="hidden" name="command" value="backup" />
  <p> <input type="radio" name="confirm" value="YES" $yes/> YES</p>
  <p> <input type="radio" name="confirm" value="NO" $no/> NO </p>
  <input type="submit" name="Submit" value="Submit" onclick="parent.upgradeTop.foo('Please Wait . . .'); this.disabled = true;"/>
</form>
ENDOF
  }
  return $doc;
}

sub c_pre
{
  my $self = shift;
  my %args = (command => 'upgrade', doc => $self->prepDoc, @_);
  my $doc = $args{doc};

  if (!$self->error) {
    $doc->print(<<"ENDOF");
  <form>
    <input type="hidden" name="app" value="$self->{app}" />
    <input type="hidden" name="state" value="DB::Upgrade" />
    <input type="hidden" name="command" value="$args{command}" />
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}" />
    <input type="submit" name="Submit" value="Upgrade" onclick="parent.upgradeTop.foo('Upgrading database.  Please Wait . . . '); this.disabled = true;"/>
  </form>
ENDOF
  }
  return $doc;
}

sub c_upgrade
{
  my $self = shift;
  my %args = (command => 'post', doc => $self->prepDoc, @_);
  my $doc = $args{doc};
  my $where = 'upgrade';

  my $upgradeFile = $self->{upgradeFile};
  # now validate we have an upgrade.psql file
  if (! -r $upgradeFile)
  {
    $self->error("Upgrade file '$upgradeFile' does not exist!  Error = '$!'.<br>\n");
  }
  else
  {
    #read whole file
    #find any lines with -- and cut the --and rest of line
    #chomp empty lines and throw away
    #join all lines with space
    #split all on ;
    #send each chunk to db and check for error
    open(IN, $upgradeFile) or $self->error("Can't read file '$upgradeFile'\n");
    my @data = <IN>;
    close(IN);
    for (my $i = 0; $i < scalar @data; $i++)
    {
      $data[$i] =~ s/(.*)--.*$/$1/;
      chomp($data[$i]);
      #!!! figure out a way to check for semi colon ; in strings and data so they can be allowed in psql files
    }
    @data = split(';', join(' ', @data));
    foreach my $chunk (@data)
    {
      chomp($chunk);
      if ($chunk !~ /^\s*$/)
      {
        $self->dbHandle->write($chunk);
#        $doc->print("<div>$chunk</div>");
        if ($self->dbHandle->error)
        {
          $doc->print("$where<div>$chunk</div>");
          $self->error($self->dbHandle->errorMessage);
          last;
        }
      }
    }
  }
  if (!$self->error)
  {
    $where = 'reports';
    #commit work up to this point because reports can be problematic.
    $self->{dbHandle}->commit;

    #names of new system reports may conflict with old names.
    my $upgradeFile = $self->{reports};
    # now validate we have an upgrade.psql file
    if (! -r $upgradeFile)
    {
      $self->error("<div>Upgrade file for reports '$upgradeFile' does not exist!  Error = '$!'.</div>\n");
      $doc->print("<div>Upgrade file for reports '$upgradeFile' does not exist!  Error = '$!'.</div>\n");
    }
    else
    {#upgrade reports
      open(REPORT, $upgradeFile) or $self->error("Can't read file '$upgradeFile'\n");
      my @report = <REPORT>;
      close(REPORT);
      foreach my $reportline (@report)
      {
#         $reportline =~ s/(.*)--.*$/$1/;
#         chomp($reportline);
        if ($reportline =~ /report_type_tb/ && $reportline !~ /^DELETE/i && $reportline !~ /^UPDATE/i)
        {
          #check name against database
          my @parse = split('VALUES',$reportline);
          my @fields = split(',',$parse[1]);
          $fields[0] =~ s/^\s*\(//g;
          my $sql = "UPDATE report_type_tb SET name = name||'custom' WHERE name = $fields[0]";
          $self->dbHandle->write($sql);
          if ($self->dbHandle->error) { $self->error("<div style=\"background-color: #cccccc;\">$reportline<br>$parse[2]<br>$fields[0]</div>".$self->dbHandle->errorMessage); }
        }
        if (!$self->error && $reportline ne '')
        {
          $self->dbHandle->write(sql => $reportline);
          if ($self->dbHandle->error) { $self->error("<div>$where</div>".$self->dbHandle->errorMessage); }
        }
      }
    }
  }

  if (!$self->error)
  {
    $self->{dbHandle}->commit;
    $doc->print(<<"ENDOF");
<p>Finished upgrading database.</p>
  <form>
    <input type="hidden" name="app" value="$self->{app}" />
    <input type="hidden" name="state" value="DB::Upgrade" />
    <input type="hidden" name="command" value="$args{command}" />
    <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}" />
    <input type="submit" name="Submit" value="Post Upgrade" onclick="parent.upgradeTop.foo('Performing post upgrade'); this.disabled = true;"/>
  </form>
ENDOF
  }
  else
  {
    $doc->print("Upgrade Failed.  You will need to restore the old database.");
    $doc->print($self->errorMessage);
  }

  return $doc;
}

sub c_post
{
  my $self = shift;
  my %args = (command => 'finish', doc => $self->prepDoc, commit => 1, @_);
  my $doc = $args{doc};
  my $upgradeFile = $self->{upgradePath} . "/post-upgrade.psql";

  if (-r $upgradeFile)
  {
    open(IN, $upgradeFile) or $self->error("Can't read file '$upgradeFile'\n");
    if (!$self->error)
    {
      my @data = split(";", join(" ", <IN> ) );
      foreach my $chunk (@data)
      {
        $self->dbHandle->write($chunk);
        if ($self->dbHandle->error)
        {
          $doc->print("<div>$chunk</div>");
          $self->error($self->dbHandle->errorMessage);
          last;
        }
      }
    }
  }
  if ($args{commit})
  {
    if (!$self->error)
    {
      $self->dbHandle->commit;
      $doc->print(<<"ENDOF");
    <form>
      <input type="hidden" name="app" value="$self->{app}" />
      <input type="hidden" name="state" value="DB::Upgrade" />
      <input type="hidden" name="command" value="$args{command}" />
      <input type="hidden" name="ledgerCode" value="$self->{input}->{ledgerCode}" />
      <input type="submit" name="Submit" value="Finish" />
    </form>
ENDOF
    }
    else
    {
      $doc->print($self->errorMessage."<hr>If the portal is working there should be two of these errors<hr>");
    }
  }
  return $doc;
}

sub c_finish
{
  my $self = shift;
  my %args = (doc => $self->prepDoc, DBVersion => $self->DBVersion, release => $self->release, @_);
  my $doc = $args{doc};
  my $sql = "UPDATE config_tb SET value = '$args{release}.$args{DBVersion}' WHERE name = 'dbversion'";
  $self->dbHandle->write($sql);
  if ($self->dbHandle->error) { $doc->print($self->dbHandle->errorMessage); }
  else {
    $self->dbHandle->commit;
    $self->{applicationObj}->unlockApp(app => $self->{appObj}->{id}, type => "c", company => $self->{companyObj}->{id});
    $doc->print(<<"ENDOF");
  <form target="_top">
    <input type="hidden" name="app" value="$self->{app}" />
    <input type="hidden" name="state" value="Main" />
    <input type="hidden" name="command" value="display" />
    <input type="submit" name="Submit" value="Main Menu" />
  </form>
ENDOF
  }
  return $doc;
}

sub c_restore
{
  my $self = shift;
  my %args = (command => 'pre', doc => $self->prepDoc, version => $self->{input}->{version}, @_);
  my $doc = $args{doc};
  my $version = $args{version};
  my $dumpDir = $self->{configObj}->{webRoot} . $self->{configObj}->{siteDir} . "/db_dump/$self->{app}/$self->{companyObj}->{code}/";
  if (defined $self->{input}->{confirm} && $self->{input}->{confirm} eq 'YES')
  {
    my $db = Portal::Accounting::DB::Main->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
    if ($db->error) { $doc->print($db->errorMessage); return $doc; }
    my $dbVersion = $db->getDBVersion;
    $doc->print(<<"ENDOF");
<form name="main">
  <p>Are you absolutely certain you want to restore this old database version '$version' and replace the current database version '$dbVersion'?
  This will NOT backup your current database which may or may not be corrupt.</p>
  <input type="hidden" name="app" value="$self->{app}" />
  <input type="hidden" name="state" value="DB::Upgrade" />
  <input type="hidden" name="command" value="restore" />
  <input type="hidden" name="version" value="$version" />
  <p>  <input type="radio" name="confirm" value="ABSOLUTELY" /> ABSOLUTELY !</p>
  <p> <input type="radio" name="confirm" value="NO" checked="1" /> NO </p>
  <input type="Submit" name="Submit" value="Submit" />
</form>
ENDOF
  }
  elsif (defined $self->{input}->{confirm} && $self->{input}->{confirm} eq 'NO')
  {
    $doc->print(<<"ENDOF");
  <form target="_top">
    <input type="hidden" name="app" value="$self->{app}" />
    <input type="hidden" name="state" value="Main" />
    <input type="hidden" name="command" value="display" />
    <input type="submit" name="Submit" value="Main Menu" />
  </form>
ENDOF
  }
  elsif (defined $self->{input}->{confirm} && $self->{input}->{confirm} eq 'ABSOLUTELY')
  {
    if (! defined $version) { $doc->print('<div class="error">No database version given to restore</div>'); }
    else
    {
      $doc->print('restore stub for '.$dumpDir."/".$version);
      #!!!drop old database
      #create New database
      #import data from file
      my $r = $self->{methods}->dbRestore($dumpDir.'/'.$version);
      if ($self->methods->error)
      {
        $self->error($self->methods->errorMessage);
        $doc->print('<div>If you are seeing this message then we feel very sorry for you.  It means that your backup did not restore.  You will need to manually restore the backup or have the system administrator do it for you.</div>');
      }
      else
      {
        $doc->print('It looks like your database was restored successfully.  You may now try to upgrade or contact the system admin if you are having trouble.');
      }
    }
  }
  else
  {
    my $VLIST = '  <select name="version" />';
    foreach my $file (glob $dumpDir."/*")
    {
      my $foo = fileparse($file);
      $VLIST .= "\n    <option value=\"$foo\">$foo</option>";
    }
    $VLIST .= '  </select>';
    $doc->print(<<"ENDOF");
<form name="main">
  Make absolutely certain you want to restore this old database version!
  This will NOT backup your current database which may or may not be corrupt.
  <input type="hidden" name="app" value="$self->{app}" />
  <input type="hidden" name="state" value="DB::Upgrade" />
  <input type="hidden" name="command" value="restore" />
  <input type="hidden" name="confirm" value="YES" />
  <p>
  $VLIST
  </p>
  <input type="Submit" name="Submit" value="Submit" />
</form>
ENDOF
  }
  return $doc;
}

=back

=cut
1;
__END__

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
