# Main.pm - The Object Class that provides a Main Object
#$Id: Main.pm,v 1.73 2006/04/07 19:39:24 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by James Pattie & Jason A. Pattie, 11/07/2000.

package Portal::Accounting::Main;
use strict;
use HTMLObject::Form;
use HTMLObject::Normal;
use Portal::Accounting::Base;
use Portal::Accounting::DB::Main;
use Portal::Accounting::Objects::Ledger;
use Portal::Data::Variables;
use Portal::Accounting::Objects::Menu;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.73 $';

# global variables.

# modify to represent the commands this state provides.
my %commands = ( new => "",
  display => "Main Menu",
  close => "Close App",
  preferences => "User Preferences",
  edit => "Edit Ledger Properties",
  new => "Create New Ledger",
  save => "Save Ledger Properties",
  confirmDelete => "Confirm Delete Ledger",
  delete => "Delete Ledger",
  lock => "Lock Application",
  unlock => "Unlock Application",
 );

=head1 NAME

Main - Object used to build a Main Menu for accounting package

=head1 SYNOPSIS

  use Portal::Accounting::Main;
  my $obj = Portal::Accounting::Main->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

Main is the Main Menu from which every other part of
the systems will be accessed.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion)

 Creates a new instance of the Portal::Accounting::Main module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the Main State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{commands} = \%commands;

  # do validation
  if (!$self->Portal::Accounting::Main::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item % getCommands(void)

  This method returns the commands hash and is only to be
  used by the registerApp method for creating the permissions,
  if using the AppStateCommandSecurity implementation.

=cut
sub getCommands
{
  return %commands;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject c_blank()

=cut
sub c_blank
{
  my $self = shift;

  return HTMLObject::Base->new;
}

=item HTMLObject c_display()

=cut
sub c_display
{
  my $self = shift;
  my %args = ( command => "display", @_ );
  my $command = $args{command};
  my $doc = $self->prepDoc;
  my $siteDir = $self->{configObj}->{siteDir};
  my $phrases = $self->{langObj};
  my $currVersion  = $Portal::Accounting::VERSION;
  my $mainHeight = "600";
  my $mainWidth = "1024";
  my $message;

  my $mainDBObj = Portal::Accounting::DB::Main->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
  # get Database version
  my $dbVersion = $mainDBObj->getDBVersion;
  my $firstTimeRun = $mainDBObj->areWeConfigured;
  if ($mainDBObj->error)
  {
    $self->error($mainDBObj->errorMessage);
    return undef;
  }

  my $lObj = Portal::Accounting::Objects::Ledger->new(dbHandle => $self->{dbHandle}, langObj => $self->{langObj});
  if ($lObj->error)
  {
    $self->error($lObj->errorMessage);
    return undef;
  }
  my @Ledgers = $lObj->selectArray;
  if ($lObj->error)
  {
    $self->error($self->errorMessage . $lObj->errorMessage);
    $message .= $self->errorMessage .  $lObj->errorMessage;
  }

  # store the Ledgers in the session. to make retrieval of data easier later on
  if ($self->{sessionObj}->setMode("write"))
  {
    if (!exists $self->{sessionObj}->{store}->{ledgers})
    {
      $self->{sessionObj}->{store}->{ledgers} = {};
    }

    foreach my $ledgerObj1 (@Ledgers)
    {
      $self->{sessionObj}->{store}->{ledgers}->{$ledgerObj1->get('code')} = $ledgerObj1->lite;
    }
  }
  else
  {
    $self->error($self->{sessionObj}->errorMessage);
    return undef;
  }

  if (!$self->{sessionObj}->setMode("read"))
  {
    $self->error($self->{sessionObj}->errorMessage);
    return undef;
  }

  # get the value of our accounting compressed cookie.
  $self->{cookieObj}->getCompressedCookies(cname => "accounting", names => [ "default_ledger" ]);
  my $defaultLedger = (exists $self->{cookieObj}->cookies->{default_ledger} ?
    $self->{cookieObj}->cookies->{default_ledger} : (scalar @Ledgers > 0 ?
    $Ledgers[0]->{name} : ""));

  $defaultLedger = $self->{input}->{ledgerCode} if (exists $self->{input}->{ledgerCode});  # it gets updated everytime the user selects to run
                            # a different ledger.

  $doc->setTitle("XIWA - ".$phrases->map("Main Menu"));

  my $url = $self->{baseURL};
  my $refreshUrl = $url . "?$self->{getInfo}&command=display";

  # calculate the Date for the user, synced to their timezone.
  my $tz = $self->{userObj}->tz;
  my $userDate = $self->{methods}->getDateOffset(date => "now", offset => "0 hours", toTZ => $tz, format => "%Y/%m/%d");

  if (!$firstTimeRun) # we aren't configured yet.
  {
    $mainDBObj->sayWeAreConfigured;
    if ($mainDBObj->error)
    {
      $self->error($mainDBObj->errorMessage);
      return undef;
    }
    $firstTimeRun = $phrases->map('firstTimeRun');
  }
  else
  {
    $firstTimeRun = "&nbsp;";
  }

  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");
//##################################33
  //funcs from Main
  //
function getLedgerCode()
{
  return document.mainForm.code.options[document.mainForm.code.selectedIndex].value;
}

function refreshWindow()
{
  window.location.reload(true);
}

function openL()
{
  var foo = new Array();
  var windowName = getTimeStamp();
  foo.app = 'Accounting';
  foo.state = 'Ledger';
  foo.command = 'open';
  foo.ledgerCode = getLedgerCode();
  foo.windowName = windowName;
  // toolbar is on because without it the tabs dont show up
  var options = ' width=$mainWidth,height=$mainHeight,scrollbars,status,location=no,menubar=no,copyhistory=no,directories=no,toolbar';
//  myPost(windowName, '', foo, options);
  myGet(windowName, '', foo, options);
}
//################################
END_OF_CODE

    # create the help JS function.
    $doc = $self->{methods}->displayJSHelper(doc => $doc, type => "help", langObj => $self->{langObj});
    if ($self->{methods}->error)
    {
      $self->error($self->{methods}->errorMessage);
      return undef;
    }

    # create the validateClose JS function.
    $doc = $self->{methods}->displayJSHelper(doc => $doc, type => "closeApp", langObj => $self->{langObj});
    if ($self->{methods}->error)
    {
      $self->error($self->{methods}->errorMessage);
      return undef;
    }

    my $menuObj = Portal::Accounting::Objects::Menu->new(methods => $self->{methods},
                  sessionObj => $self->{sessionObj}, langObj => $self->{langObj}, userObj => $self->{userObj},
                  input => $self->{input}, portalSession => $self->{portalSession});
    if ($menuObj->error)
    {
      $self->error($menuObj->errorMessage);
      return undef;
    }

    my $menuString = $menuObj->print(location => "menubar", state => "Main", command => $command, doc => $doc);
    if ($menuObj->error)
    {
      $self->error($menuObj->errorMessage);
      return undef;
    }

  my $newl = $phrases->map("New");
  my $edit = $phrases->map("Edit");
  my $open = $phrases->map("Open");
  my $ledgersFound = "        ".$phrases->map("Ledgers")." ".$phrases->map("found").": ".@Ledgers.".\n";
  $doc->setFocus("body");
  $doc->print(<<"END_OF_CODE");
$menuString
<div style="float: left;">
  Database Version = <b>$dbVersion</b> | HTMLObject Version = <b>$HTMLObject::Normal::VERSION</b>
  | $firstTimeRun
</div>
<div style="float: right">
  $userDate
</div>
<div class="title">XIWA $currVersion</div>
<div class="secondary" style="margin: auto; width: 30em; text-align: center; ">
  $ledgersFound
  <div class="primary">
END_OF_CODE
  if (scalar @Ledgers > 0)
  {
    $doc->print(<<"END_OF_CODE");
    <form name="mainForm" method="post" style="float: left; padding-right: 1em;">
      <select name="code">
END_OF_CODE
    foreach my $ledgerObj (@Ledgers)
    {
      if ($ledgerObj->isValid)
      {
        my $str = "<option value=\"" . $ledgerObj->get('code') . "\"";
        $str .= " selected" if ($ledgerObj->get('code') eq $defaultLedger);
        $str .= ">" . $ledgerObj->get('name');
        $doc->print("          $str\n");
      }
      else
      {
        $message .= $phrases->map("Ignoring_invalid") . " ledgerObject: ".$ledgerObj->get('name')."<br>\n";
        $message .= $ledgerObj->errorMessage;
      }
    }
    $doc->print(<<"END_OF_CODE");
      </select>
      <input type="button" name="openMe" value="$open" onclick="openL();">
      
      <input type="hidden" name="app" value="Accounting" />
      <input type="hidden" name="state" value="Main" />
      <input type="hidden" name="command" value="edit" />
      <input type="submit" name="edit" value="$edit" />
    </form>
END_OF_CODE
  }
  $doc->print(<<"END_OF_CODE");
    <form name="newForm" style="text-align:left">
      <input type="submit" name="new" value="$newl" />
      <input type="hidden" name="app" value="Accounting" />
      <input type="hidden" name="state" value="Main" />
      <input type="hidden" name="command" value="new" />
    </form>
  </div>
</div>  
<div style="clear:both; text-align: center; margin: 2em; ">
END_OF_CODE
#  $doc->print("      <a href=\"\" onclick=\"javascript: {window.open('$url?app=Accounting&state=Main&command=preferences', '$phrases->{Main_Preferences}', 'width=800,height=600,scrollbars'); return false; }\">".$phrases->map("Preferences")."</a>");
  my $Accounts = $phrases->map("Accounts");
  $doc->print("      <a href=\"$url?app=Accounting&state=Accounts&command=display\" target=\"$Accounts\" onclick=\"javascript: {window.open('$url?app=Accounting&state=Accounts&command=display', '$Accounts', 'width=800,height=600,scrollbars,toolbar=no,location=no,menubar=no,copyhistory=no,directories=no'); return false; }\">".$phrases->map("Chart of Accounts")."</a>");
  
  if ($self->{userObj}->{admin})
  {
    $doc->print("      | <a href=\"$url?app=$self->{app}&state=DB::Upgrade&command=restore\" />".$phrases->map("Restore Database")."</a>");
    if ($self->{applicationObj}->isAppLocked(app => $self->{appObj}->{id}) )
    {
      $doc->print("      | <a href=\"$url?app=$self->{app}&state=Main&command=unlock\" />".$phrases->map("Unlock Application")."</a>");
    }
    else
    {
      $doc->print("      | <a href=\"$url?app=$self->{app}&state=Main&command=lock\" />".$phrases->map("Lock Application")."</a>");
    }
  }
  $doc->print(<<"END_OF_CODE");
</div>
END_OF_CODE

  $doc->print("<div class=\"message\" style=\"color: red;\">$message</div>") if ($message);

  # don't forget to reset the accounting cookie.
  my $datestr = $self->{methods}->getDateOffset(date => "now", offset => "30 days", toTZ => "GMT", format => "%a, %d-%b-%Y 00:00:00 GMT");
  $doc->setCompressedCookie(name => "accounting", cookies => [ ["default_ledger", $defaultLedger] ], expires => $datestr, domain => $self->{configObj}->cookieDomain);

  return $doc;
}

=item HTMLObject c_close()

=cut
sub c_close
{
  my $self = shift;
  my $doc;

  $doc = $self->processCloseAppEvent(langEntry => "Logout");
  if ($self->error)
  {
    $self->prefixError();
    return undef;
  }

  return $doc;
}

=item HTMLObject c_preferences()

=cut
sub c_preferences
{
  my $self = shift;
  my $doc = HTMLObject::Normal->new;

  $doc->setFocus("body");
  $doc->print("<h2>These will be user defined preferences such as window sizes etc.</h2>\n");
  return $doc;
}

sub c_delete
{
  my $self = shift;
  $self->{OBJECT} = Portal::Accounting::Objects::Ledger->new(langObj => $self->langObj, dbHandle => $self->dbHandle, windowName => $self->{input}->{windowName});
  if ($@) { $self->error($@); return undef;  }
  if ($self->{OBJECT}->error) { $self->error($self->{OBJECT}->errorMessage); return undef;  }
  return $self->SUPER::c_delete(@_);
}

sub c_edit
{
  my $self = shift;
  $self->{OBJECT} = Portal::Accounting::Objects::Ledger->new(langObj => $self->langObj, dbHandle => $self->dbHandle, windowName => $self->{input}->{windowName});
  if ($@) { $self->error($@); return undef;  }
  if ($self->{OBJECT}->error) { $self->error($self->{OBJECT}->errorMessage); return undef;  }
  my $doc = $self->SUPER::c_edit(@_);
  $doc->print($self->mainButton);
  return $doc;
}

sub c_save
{
  my $self = shift;
  $self->{OBJECT} = Portal::Accounting::Objects::Ledger->new(langObj => $self->langObj, dbHandle => $self->dbHandle, windowName => $self->{input}->{windowName});
  if ($@) { $self->error($@); return undef;  }
  if ($self->{OBJECT}->error) { $self->error($self->{OBJECT}->errorMessage); return undef;  }
  $self->{autoSubmit} = 0;
  my $doc = $self->SUPER::c_save(@_);
  my %form;
  if ($self->error) { $doc->print($self->errorMessage); $self->resetError; }
  else
  {
    my $url = '?app='.$self->{app}.'&state=Accounts&command=display&ledgerCode='.$self->{OBJECT}->get('code');
    $doc->print('<br><br><a target="_new" href="'.$url.'" onclick="window.open(\''.$url.'\', \'_new\', \'toolbar=no,copyhistory=no\'); return false;">You should now assign accounts to this ledger.</a>');
  }
  return $doc;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
