# Account.pm - The Object Class that provides a Account Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: Account.pm,v 1.56.4.1 2006/07/25 15:54:27 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Account;
use strict;
use Portal::Accounting::Objects::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.56.4.1 $';

=head1 NAME

Account - Object used to create, edit, save an Account.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Account;
  my $accountObj = Portal::Accounting::Objects::Account->new();
  $populate(
    name => "test", number => "1234", type => 1,
    code => "1234", taxable => "true");
  if ($error())
  {
    die $errorMessage();
  }

=head1 DESCRIPTION

Account is a Account class.  This represents an account in the
accounting system.  Accounts are grouped by their type and must have
unique names and numbers.  Right now number and code are the same but
eventually code will be user defined and number will be internally derived.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(dbHandle => , langObj =>)

 Creates a new instance of the Portal::Accounting::Objects::Account module.

=cut
sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( meta => "", type => '', code => '', name => '', ledgerCode => "", embed => 0, @_ );

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }
  # instantiate anything unique to this module
  $self->{'table'} = "account_tb";
  my $l = $self->{langObj};
  my $Submit = $l->map("Save");
  $self->{data}->{main} = {
    "id" => { -Label => $l->map("Id"), -Type => "hidden", -Value => "", -DataType => "int" },
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Accounts" },
    "command" => { -Type => "hidden", -Value => "save" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode}, -DataType => "string" },
    "name" => { -Label => $l->map("Name"), -Type => "text", size => 20, maxlength => 30, -DataType => "string", -Value => $args{name},
      onblur => "if (this.form.code.value == '') { this.form.code.value = suggestCode(this.form.name.value, this.form.type.value); }",
#      -onload => 'this.select();',
    },
    "code" => { -Label => $l->map("Code"), -Type => "text", size => 20, maxlength => 30, -Value => $args{code}, -DataType => "string" },
    "taxable" => { -Label => $l->map("Taxable"), -Type => "checkbox", -Boolean => 1, -DataType => "boolean", },
    "type" => { -Label => $l->map("Type"), -Type => "select", -DataType => "int",
      -Options => $self->getAccountTypes4Select(meta => $args{'meta'}, noGainLoss => 1),
      -Value => $args{type},
    },
    "meta" => { -Value => $args{meta}, -Type => "hidden", -DataType => "int" },
    "nosubs" => { -Value => 0, -Type => "hidden" },
    "debitIncreases" => { -Type => "hidden" },
    "embed" => { -Value => $args{embed} , -Type => "hidden", -DataType => "int" },
  };
  if ($args{meta} > 0 && $args{meta} < 4) { $self->{data}->{main}->{taxable}->{disabled} = 1;  }
  else { $self->{data}->{main}->{taxable}->{checked} = 1;  }
  if ($args{type} ne '') { $self->{data}->{main}->{type}->{-ReadOnly} = 1;}
  $self->{css}->{main} = <<"ENDOF";
ENDOF
  $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS#>
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  #FIELD=id#
  #FIELD=meta#
  #FIELD=nosubs#
  #FIELD=embed#
  <table class="primary">
    <tr>
      <td>#LABEL=type#</td><td>#FI=type#</td>
    </tr>
    <tr>
      <td>#LABEL=name#</td><td>#FI=name#</td>
    </tr>
    <tr>
      <td>#LABEL=code#</td><td>#FI=code#</td>
    </tr>
    <tr>
      <td>#LABEL=taxable#</td><td>#FI=taxable#
    </tr>
  </table>
  <input type="submit" name="Submit" value="$Submit">
</form>
ENDOF
  $self->{javascript}->{main} = <<"ENDOF";
var firstDigits;
function setCode(myForm)
{
  var type = myForm.type[myForm.type.selectedIndex].value;
  myForm.code.value = firstDigits[type];
}
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( name code ) ],
    optional => [ qw( id taxable embed ) ],
    constraints => {
      id => qr/^\d\d+$/,
      name => qr/^[\d\w][\d\s\w-,]*$/,
      code => qr/^[\d\w.-]+$/,
    },
  };

  $self->{defaultForm} = "main";
  ##db infrastructure
  $self->{id} = "id";
  $self->{INSERTorder}[0] = [ qw( name code type taxable ) ] ;  #insert order for db column names
  $self->{INSERTorder}[1] = [ qw( first_digit nosubs ) ] ;  #insert order for db column names
  $self->{newOnly}[0] = { number => "" };
  $self->{db2data}[0] = { number => 'id' };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[0] = {reverse %{$self->{db2data}[0]}};
  $self->{db2data}[1] = { first_digit => 'meta', debit_increases => 'debitIncreases' };  #map database columns to ->data hash, not specified assumes same name
  $self->{data2db}[1] = {reverse %{$self->{db2data}[1]}};

  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM account_tb WHERE number = #FIELD=id#";
  $self->{SELECT}[1] = "SELECT first_digit, nosubs, debit_increases FROM acct_type_tb WHERE num = (SELECT type FROM account_tb WHERE number = #FIELD=id#)";
  $self->{SELECTID}[0] = "SELECT number FROM account_tb WHERE name = '#FIELD=name#' AND code = '#FIELD=code#'";
  $self->{INSERT}[0] = "INSERT INTO account_tb (number, ##COLUMNS##) VALUES ( ";
  $self->{INSERT}[0] .= "  next_account( ";
  $self->{INSERT}[0] .= "    (SELECT MAX(account_tb.number) FROM account_tb, acct_type_tb WHERE account_tb.type = acct_type_tb.num AND acct_type_tb.first_digit = '#FIELD=meta#'))";
  $self->{INSERT}[0] .= ", ##VALUES##)";
  $self->{UPDATE}[0] = "UPDATE account_tb SET ##UPDATE## WHERE number = #FIELD=id# AND ( (SELECT nosubs FROM acct_type_tb WHERE num = type) = false OR type = '#FIELD=type#' )";
  #remember to use single quotes around ##foo## if needed
  $self->{DELETE}[0] = "DELETE FROM account_tb WHERE number = #FIELD=id#";  #is 'id' a hardcoded form element?
  #find out ifis this being referenced from other tables before deleting
  $self->{preDELETE}[0] = "SELECT debitcredit_tb.parent_num FROM debitcredit_tb WHERE debitcredit_tb.account_num = #FIELD=id#";
  #what should we output as the value of the thing just deleted?
  $self->{postDELETE}[0] = "SELET name||', '||code||', '||number FROM account_tb WHERE number = '#FIELD=id#'";
  if ($args{ledgerCode} eq '')
  {
    $self->{LIST}[0] = "SELECT name, number AS id FROM account_tb ORDER BY name";
  }
  else
  {
    $self->{LIST}[0] = "SELECT account_tb.name, account_tb.number AS id FROM account_tb, local_accounts_tb WHERE local_accounts_tb.ledger_code = '#FIELD=ledgerCode#' AND account_tb.number = local_accounts_tb.acct_num ORDER BY account_tb.name";
  }
  return $self;
}

=item %Accounts selectTypeHash

  returns a hash of {name => 'Expense', meta => '5' } for each account type in acct_type_tb with the key as the type number/id

=cut
sub selectTypeHash
{
  my $self = shift;
  my $sql = "SELECT name, first_digit AS meta, num AS id FROM acct_type_tb ORDER BY name";
  my %data;
  my $sth = $self->dbHandle->read($sql);
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); return undef; }
  while (my $info = $sth->fetchrow_hashref)
  {
    $data{$info->{id}} = {name => $info->{name}, meta => $info->{meta} };
  }
  return %data;
}

=item %Accounts selectTypeHash

  returns a hash of {name => 'Expense', meta => '5' } for each account type in acct_type_tb with the key as the type number/id

=cut
sub selectTypeArray
{
  my $self = shift;
  my $sql = "SELECT name, first_digit AS meta, num AS id FROM acct_type_tb ORDER BY name";
  my @data;
  my $i = 1;
  my $sth = $self->dbHandle->read($sql);
  if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); return undef; }
  while (my $info = $sth->fetchrow_hashref)
  {
    $data[$i++] = {name => $info->{name}, meta => $info->{meta}, type => $info->{id} };
    $data[0]{$info->{id}} = $i;
  }
  $self->{TYPEARRAY} = $sql;
  return @data;
}

=item @Accounts selectArrayByType

=cut
sub selectArrayByType
{
  my $self = shift;
  my $temp = $self->{LIST}[0];
  $self->{LIST}[0] = "SELECT account_tb.name, account_tb.number AS id FROM account_tb, local_accounts_tb WHERE local_accounts_tb.ledger_code = '#FIELD=ledgerCode#' AND account_tb.number = local_accounts_tb.acct_num AND account_tb.type = #FIELD=type# ORDER BY account_tb.name";
  my @foo = $self->SUPER::selectArray(@_);
  $self->{LIST}[0] = $temp;
  return @foo;
}

=item @Accounts selectArrayByMeta

=cut
sub selectArrayByMeta
{
  my $self = shift;
  my $temp = $self->{LIST}[0];
  $self->{LIST}[0] = "SELECT account_tb.name, account_tb.number AS id FROM account_tb, local_accounts_tb WHERE local_accounts_tb.ledger_code = '#FIELD=ledgerCode#' AND account_tb.number = local_accounts_tb.acct_num AND acct_type_tb.num = account_tb.type AND acct_type_tb.first_digit = #FIELD=meta# ORDER BY account_tb.name";
  my @foo = $self->SUPER::selectArray(@_);
  $self->{LIST}[0] = $temp;
  #!!!
  $self->error($temp);
  return @foo;
}

=item scalar selectOptionsByType

  set the type with ->set('type, $type); before calling

=cut
sub selectOptionsByType
{
  my $self = shift;
  my $temp = $self->{LIST}[0];
  $self->{LIST}[0] = "SELECT account_tb.name, account_tb.number AS id FROM account_tb, local_accounts_tb WHERE local_accounts_tb.ledger_code = '#FIELD=ledgerCode#' AND account_tb.number = local_accounts_tb.acct_num AND account_tb.type = #FIELD=type# ORDER BY account_tb.name";
  my $foo = $self->SUPER::selectOptions(@_);
  $self->{LIST}[@{$self->{LIST}}] = $self->{LIST}[0];
  $self->{LIST}[0] = $temp;
  return $foo;
}

=item scalar selectOptionsByMeta

=cut
sub selectOptionsByMeta
{
  my $self = shift;
  my $temp = $self->{LIST}[0];
  $self->{LIST}[0] = "SELECT account_tb.name, account_tb.number AS id FROM account_tb, local_accounts_tb, acct_type_tb WHERE local_accounts_tb.ledger_code = '#FIELD=ledgerCode#' AND account_tb.number = local_accounts_tb.acct_num AND account_tb.type = acct_type_tb.num AND acct_type_tb.first_digit = '#FIELD=meta#' ORDER BY account_tb.name";
  my $foo = $self->SUPER::selectOptions(@_);
  $self->{LIST}[0] = $temp;
  return $foo;
}

=item %Options selectOptions(name|code|both)

  This routine calls the base select Options but can be toggled to select names or codes for the list of accounts.  Defaults to name

=cut
sub selectOptions
{
  my $self = shift;
  my $which = shift;
  my $led = "";
  my $l = $self->get("ledgerCode");
  if ($l ne '') { $led = " AND local_accounts_tb.ledger_code = '$l' "; }
  if ($which eq 'both')
  {
    $self->{LIST}[0] = "SELECT account_tb.name||'-'||account_tb.code AS name, account_tb.number AS id FROM account_tb, local_accounts_tb WHERE local_accounts_tb.acct_num = account_tb.number $led ORDER BY name";
  }
  elsif ($which eq 'code')
  {
    $self->{LIST}[0] = "SELECT account_tb.code AS name, account_tb.number AS id FROM account_tb, local_accounts_tb WHERE local_accounts_tb.acct_num = account_tb.number $led ORDER BY name";
  }
  else
  {
    $self->{LIST}[0] = "SELECT account_tb.name AS name, account_tb.number AS id FROM account_tb, local_accounts_tb WHERE local_accounts_tb.acct_num = account_tb.number $led ORDER BY name";
  }
  return $self->SUPER::selectOptions(@_);
}

=item HTMLObject::Form::Options getAccountType4Select(meta => , nosubs => )

  This routine selects the account types from the database and creates select options to put on an HTMLObject::Form select element.
  meta: 1 - 5 will limit to Assets (1), Liabilities(2), etc. empty string will gather all types
  nosubs: defaults to 0 which will include special acct_types,  1 will cause special accounts to not show up.
    this is to help keep accounts already used from being changed into special accounts
    ex.  changing an AR into bank/cash because that would cause many problems.

=cut
sub getAccountTypes4Select
{
  my $self = shift;
  my $l = $self->{langObj};
  my %args = (meta => "", nosubs => 0, noGainLoss => 0, @_);
  my $nosubs = $args{nosubs};
  my $meta = $args{meta};
  my $gl;
  my $where = "";
  if ($meta ne "")
  {
    if ($meta =~ /^[1-5]$/)
    {
      $meta = " first_digit = '$meta' ";
    }
    else
    {
      $self->error("Invalid meta for getAccountTypes4Select<br>\n");
      return undef;
    }
    $where = " WHERE ";
  }
  if ($nosubs)
  {
    if ($where ne '') { $nosubs = " AND "; }
    $nosubs .= " nosubs = false ";
    $where = " WHERE ";
  }
  else { $nosubs = ""; }
  if ($args{noGainLoss})
  {
    if ($where ne '') { $gl = " AND "; }
    $gl .= ' num <> 8 AND num <> 18 ';
    $where = " WHERE ";
  }
  my $sql = "SELECT name, num, first_digit FROM acct_type_tb $where $meta $nosubs $gl ORDER BY first_digit, name";
  my $db = $self->{dbHandle};
  my $result = $db->read($sql);
  if ($db->error) { $self->error($db->errorMessage); return undef; }
  #walk this data and translate $l->map($d->name)
  #pull out first_digit
  my @foo;
  while (my $d = $result->fetchrow_arrayref)
  {
    #lang Map the meta types based on first_digit.  put in $pre
    my $pre = "";
    if ($d->[2] == 1) {  $pre = $l->map("Asset");  }
    elsif ($d->[2] == 2) {  $pre = $l->map("Liability");  }
    elsif ($d->[2] == 3) {  $pre = $l->map("Equity");  }
    elsif ($d->[2] == 4) {  $pre = $l->map("Revenue");  }
    elsif ($d->[2] == 5) {  $pre = $l->map("Expense");  }
    else {  $self->error("Invalid first_digit returned from record: ".$d->[0].", ".$d->[1].", ".$d->[2]."\n");  return undef;  }
    $pre .= ":";  #add seperator
    #add pre to each type with lang map
    $d->[0] = $pre . $l->map($d->[0]);
    push (@foo, [$d->[0], $d->[1]]);
    $self->{firstDigit}->{$d->[1]} = $d->[2];
    $self->{types}->{$d->[1]} = $d->[0];
  }
  return $self->{formObj}->createSelectOptions(\@foo) ;
}

=item string suggestCode(code)

  This routine suggests a code for the account by putting them into a javascript array
  Optional:  code - 1-5 to return the next value for Asset (1), Liability(2), and so on

=cut
sub suggestCode
{
  my $self = shift;
  my $code = shift;
  if ($code !~ /^[1-5]$/) { $code = ""; }
  my $sql = "SELECT acct_type_tb.first_digit AS f, next_account(MAX(account_tb.number)) AS n WHERE account_tb.type = acct_type_tb.num GROUP BY acct_type_tb.first_digit";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef; }
  while (my $data = $result->fetchrow_hashref)
  {
    #if we roll to the next first_digit range increment by a factor of 10
    my $next = $data->{n};
    #use this if ever making an advanced create account that shows all types
    $self->{javascript}->{main} .= << "ENDOF";
  firstDigits[$data->{f}] = $next;
ENDOF
    if ($code == $data->{f}) { $code = $next;  }
  }
  return $code;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub firstDigit
{
  my $self = shift;
  my $type = shift;
  my $f = "";
  my $sql = "SELECT first_digit FROM acct_type_tb WHERE num = '$type'";
  my $result = $self->{dbHandle}->read($sql);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef; }
  if (my $data = $result->fetchrow_hashref)
  {
    $f = $data->{first_digit};
  }
  else { $self->error("Invalid type. no first_digit returned for: $type");  return undef;  }
  return $f;
}

=item string printXML(fullXML bool, indent int)

 This routine returns an XML snippet for this object for use in such
 things as exporting data

 fullXML - defaults to 0 (false).  If true, we output the xml header tag.
 indent - defaults to 2.  Specifies how many spaces in our top level tag
          needs to be.  If fullXML is true, then we force 0, otherwise we
          use what the user specifies.

=cut

sub populate
{
  my $self = shift;
  my $result = $self->SUPER::populate(@_);
  if ($self->{data}->{main}->{nosubs}->{-Value})
  {
    $self->{data}->{main}->{type}->{-ReadOnly} = 1;
  }
  my $o = $self->getAccountTypes4Select(meta => $self->get('meta'));
  if ($self->error) { return undef; }
  $self->{data}->{main}->{type}->{-Options} = $o;
  return $result;
}

sub select
{
  my $self = shift;
  my $result = $self->SUPER::select(@_);
  my $o;
  if ($self->{data}->{main}->{nosubs}->{-Value})
  {
    $o = $self->getAccountTypes4Select(meta => $self->get('meta'));
    $self->{data}->{main}->{type}->{-ReadOnly} = 1;
  }
  else
  {
    $o = $self->getAccountTypes4Select(meta => $self->get('meta'), nosubs => 0);
  }
  if ($self->error) { return undef; }
  $self->{data}->{main}->{type}->{-Options} = $o;
  return $result;
}

=item string bankOverhead(ledgerCode)

  this routine returns a string with all of the javascript overhead needed
  for incrementing bank account check and deposit numbers

=cut
sub bankOverhead
{
  my $self = shift;
  my $lc = shift;
  my $j = "";
  my $db = $self->{dbHandle};
  my $banks = "SELECT next_check_deposit('$lc', account_tb.number, false) AS check, next_check_deposit('$lc', account_tb.number, true) AS deposit, account_tb.number AS id FROM account_tb, local_accounts_tb";
  $banks .= " WHERE account_tb.type = 0 AND local_accounts_tb.ledger_code = '$lc' AND account_tb.number = local_accounts_tb.acct_num";
  my $result = $db->read($banks);
  if ($db->error) { $self->error($db->errorMessage); return undef; }
  while (my $d = $result->fetchrow_hashref)
  {
    if ($d->{check} ne '') {  $j .= "BANKS[CHECK][$d->{id}] = \"$d->{check}\"; ";  }
    if ($d->{deposit} ne '') {  $j .= "BANKS[DEPOSIT][$d->{id}] = \"$d->{deposit}\"; ";  }
  }
  return $j;
}

=item string javascriptOverhead()

  this routine returns a string with all of the javascript overhead needed
  for managing accounts such as types, names, lists of accounts, links to sub types, etc.

=cut
sub javascriptOverhead
{
  my $self = shift;
  my $l = $self->langObj;
  my $j;
  my $k;
  my $db = $self->{dbHandle};
  #typeNames
  #links
  #bank account numbers
  #types
  #names and codes
  my $sql = "SELECT account_tb.name AS name, account_tb.taxable AS taxable,";
  $sql   .= " account_tb.code AS code, account_tb.type AS type, account_tb.number AS id,";
  my $ledger = $self->get('ledgerCode');
  if ($ledger ne "")
  {
    $sql   .= " (SELECT local_accounts_tb.acct_num FROM local_accounts_tb WHERE local_accounts_tb.acct_num = account_tb.number AND ledger_code = '$ledger' LIMIT 1) AS active";
  }
  else
  {
    $sql   .= " NULL ";
  }
  $sql   .= " FROM account_tb ORDER BY account_tb.name";
  my $typeNames = "SELECT name, num, first_digit, sub_acct_type FROM acct_type_tb;";
  my $result = $db->read($sql);
  if ($db->error) { $self->error($db->errorMessage); return undef; }
  while (my $d = $result->fetchrow_hashref)
  {
    my $tax = "";
    if ($d->{taxable}) { $tax = "*"; }
    $j .= "ACCOUNTTYPES[$d->{id}] = $d->{type};\n";
    if ($d->{active} ne "")
    {
      $j .= "ACCOUNTS['name']['$d->{name}$tax'] = $d->{id};\n";
      $j .= "ACCOUNTS['code']['$d->{code}$tax'] = $d->{id};\n";
      if ($d->{type} == 0)
      {
        $j .= "BANKACCOUNTS['$d->{name}'] = $d->{id};\n";
      }
      elsif ($d->{type} == 111)
      {
        $j .= "RECEIVABLES['$d->{name}'] = $d->{id};\n";
      }
      elsif ($d->{type} == 5)
      {
        $j .= "PAYABLES['$d->{name}'] = $d->{id};\n";
      }
      elsif ($d->{type} == 7)
      {
        $j .= "EQUITYACCOUNTS['$d->{name}'] = $d->{id};\n";
      }
    }
    else
    {
      $j .= "INACTIVE_ACCOUNTS['name']['$d->{name}$tax'] = $d->{id};\n";
      $j .= "INACTIVE_ACCOUNTS['code']['$d->{code}$tax'] = $d->{id};\n";
      if ($d->{type} == 0)
      {
        $j .= "INACTIVE_BANKACCOUNTS['$d->{name}'] = $d->{id};\n";
      }
      elsif ($d->{type} == 111)
      {
        $j .= "INACTIVE_RECEIVABLES['$d->{name}'] = $d->{id};\n";
      }
      elsif ($d->{type} == 5)
      {
        $j .= "INACTIVE_PAYABLES['$d->{name}'] = $d->{id};\n";
      }
      elsif ($d->{type} == 7)
      {
        $j .= "INACTIVE_EQUITYACCOUNTS['$d->{name}'] = $d->{id};\n";
      }
    }
  }
  $j .= "\n";
  #walk this data and translate $l->map($d->name)
  my $result = $db->read($typeNames);
  if ($db->error) { $self->error($db->errorMessage); return undef; }
  while (my $d = $result->fetchrow_hashref)
  {
     $j .= "ACCOUNTTYPENAMES[$d->{first_digit}][$d->{num}] = '".$l->map($d->{name})."';\n";
     $k .= "ACCOUNTLINKS[$d->{num}] = new Array();\n";
     my @subs = split(",", $d->{sub_acct_type});
     for (my $i = 0; $i < @subs; $i++)
     {
       $k .= "ACCOUNTLINKS[$d->{num}][$i] = ".$subs[$i].";\n";
     }
  }
  $j .= $k."\n";
  $j .= $self->bankOverhead($ledger);
  return $j."\n";
}

=item integer assignAccounts(ledgerCode => , accounts => \@accounts)

  This routine will assign the given accounts to the given ledger.
  ledgerCode - string
  accounts - an arrayref to array of account numbers

=cut
sub assignAccounts
{
  my $self = shift;
  my %args = ( ledgerCode => "", accounts => undef, @_ );
  my $l = $args{ledgerCode};
  my $a = $args{accounts};
  my $records = 0;
  if ($l eq "") { $self->error("Invalid ledgerCode in assignAccounts<br>\n");  return undef;  }
  if (! defined $a) { $self->error("No accounts given in assignAccounts<br>\n");  return undef;  }
  foreach my $account (@{$a})
  {
    if ($account !~ /^\d+$/) { $self->error("Invalid account given in assignAccounts: '$account'<br>\n");  return undef;  }
    my $sql = "DELETE FROM local_accounts_tb WHERE ledger_code = ? AND acct_num = ?";
    my @plug = ( $l, $account );
    my $result = $self->{dbHandle}->write(sql => $sql, plug => \@plug);
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{dbHandle}->errorMessage);
      $self->{dbHandle}->rollback;
      return undef;
    }
    $sql = "INSERT INTO local_accounts_tb (acct_num, ledger_code, taxable) VALUES (?, ?, ";
    $sql .= " (SELECT taxable FROM account_tb WHERE number = ?)";
    $sql .= ")";
    my @plug = ( $account, $l, $account );
    my $result = $self->{dbHandle}->write(sql => $sql, plug => \@plug);
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{dbHandle}->errorMessage);
      $self->{dbHandle}->rollback;
      return undef;
    }
    $records++;
  }
  $self->{dbHandle}->commit;
  return $records;
}

=item integer unAssignAccounts(ledgerCode => , accounts => \@accounts)

  This routine will unassign the given accounts from the given ledger.
  ledgerCode - string
  accounts - an arrayref to array of account numbers

=cut
sub unAssignAccounts
{
  my $self = shift;
  my %args = ( ledgerCode => "", accounts => undef, @_ );
  my $l = $args{ledgerCode};
  my $a = $args{accounts};
  my $records = 0;
  if ($l eq "") { $self->error("Invalid ledgerCode in assignAccounts<br>\n");  return undef;  }
  if (! defined $a) { $self->error("No accounts given in assignAccounts<br>\n");  return undef;  }
  foreach my $account (@{$a})
  {
    if ($account !~ /^\d+$/) { $self->error("Invalid account given in assignAccounts: '$account'<br>\n");  return undef;  }
    my $sql = "DELETE FROM local_accounts_tb WHERE ledger_code = '$l' AND acct_num = '$account'; ";
    my $result = $self->{dbHandle}->write(sql => $sql);
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{dbHandle}->errorMessage);
      $self->{dbHandle}->rollback;
      return undef;
    }
    $records++;
  }
  $self->{dbHandle}->commit;
  return $records;
}

=item integer insert()

  Routine that tries to take data in object and
  inserts it into the database from the main data hash
  optional: string - name of a different data hash to take data from
  returns: undef for error, number of records on success

  how do I know what the id of the object is that was just inserted?
  I can maybe use the $self->{id} field and look for a nextval?

=cut
sub insert { my $self = shift; return $self->insertUpdate('insert', @_); }
sub update { my $self = shift; return $self->insertUpdate('update', @_); }
sub insertUpdate
{
  my $self = shift;
  my $which = shift;
  my $form = 'main';
#   check for duplicates on unique fields
#   if found return problem condition
  my $checked = ($self->{data}->{$form}->{taxable}->{checked} == 1 ?'true':'false');
  my $update = ($which eq 'update'?' AND account_tb.number <> '.$self->get('id').' ':'');
  my $CodeAlreadyExists = $self->{langObj}->map('AlreadyExists').' '.$self->{langObj}->map("code")
    .". ".$self->{langObj}->map("need2assign");
  my $sq1 = "SELECT number FROM account_tb WHERE account_tb.code = '".$self->get('code')."'".$update;
  my $NameAlreadyExists = $self->{langObj}->map('AlreadyExists').' '.$self->{langObj}->map("name")
    .". " . $self->{langObj}->map("AccountConstraints") . $self->{langObj}->map("need2assign");
  my $sq2 = "SELECT number FROM account_tb WHERE account_tb.name = '".$self->get('name')."' AND account_tb.taxable = '$checked'".$update;
  my $result = $self->{dbHandle}->read($sq1);
  if ($self->{dbHandle}->error)
  {
    $self->error("code");
    $self->invalid('code','',$self->{dbHandle}->errorMessage."<br>\n\n$sq1");
    return -1;
  }
  if (my $d = $result->fetchrow_arrayref)
  {
    $self->error("code");
    $self->invalid("code", $self->get('code'), $CodeAlreadyExists);
    return -1;
  }
  $result = $self->{dbHandle}->read($sq2);
  if ($self->{dbHandle}->error)
  {
    $self->error("name");
    $self->invalid("name",'',$self->{dbHandle}->errorMessage."<br>\n\n$sq2");
    return -1;
  }
  if (my $d = $result->fetchrow_arrayref)
  {
    $self->error("name");
    $self->invalid("name", $self->get('name'), $NameAlreadyExists);
    return -1;
  }
  eval('return $self->SUPER::'.$which.'(@_);');
}

sub isNew
{
  my $self = shift;
  if ($self->get('id') eq '') { return 1; }
  return 0;
}

sub printForm
{
  my $self = shift;
  my %r = $self->SUPER::printForm(@_);
  if (!$self->error && defined %r)
  {
    my $meta = $self->get('meta');
    my $limit;
    if ($meta ne '') { $limit = "WHERE first_digit = $meta"; }
    my $out = '';
    #add help messages to page.
    my $sql = "SELECT first_digit AS meta, num AS id FROM acct_type_tb $limit ORDER BY first_digit, name";
    my $sth = $self->dbHandle->read($sql);
    if ($self->dbHandle->error) { $self->error($self->dbHandle->errorMessage); return undef; }
    while (my $info = $sth->fetchrow_hashref)
    {
      $out .= '<div id="h'.$info->{meta}.'_'.$info->{id}.'" class="metaHelp">'.$self->langObj->map("metaHelp_$info->{meta}_$info->{id}")."</div>\n";
    }
    $r{body} .= '<div class="tertiary" style="margin: .5em;">'.$out."</div>\n";
    $r{css} .= ".metaHelp { margin: .5em; padding: .2em;} \n";
  }
  return %r;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::Base(3)

=cut
