# Base.pm - The Object Class that provides a Base Object
#$Id: Base.pm,v 1.76.4.2 2006/08/01 23:09:38 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::Base;
use strict;
use Portal::Objects::Base;
use Portal::Data::Variables;
use Date::Calc qw(Today);

use vars qw($AUTOLOAD $VERSION @ISA @EXPORT %Elements);

require Exporter;

@ISA = qw(Portal::Objects::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.76.4.2 $';
%Elements = ();

=head1 NAME

BaseObject - Base Object for deriving other objects.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::Base;
  @ISA = qw(Portal::Accounting::Objects::Base Exporter AutoLoader);

=head1 DESCRIPTION

BaseObject is a Base class for Accounting Objects such as
Objects::Ledger and Objects::Account.  It contains methods for
handling database select, insert, update, delete.  Also contains
methods for handling forms which are defined in the child objects.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::Base.
    See Portal::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( imageDir => '/portal/images/Accounting/', debug => 0, dbHandle => undef, @_ );
  $self->{name} = "Portal::Accounting::Objects::Base";
  $self->{dbHandle} = $args{dbHandle};
  if (! defined $self->{dbHandle}) { $self->error("<p>missing dbHandle</p>\n"); }
  $self->{debug} = $args{debug};
  $self->{debugString} = "";
  $self->{warningString} = "";
  $self->{action} = "";
  $self->{method} = "post";
  ##db infrastructure
  $self->{id} = "";
  $self->{table} = '';
  $self->{column} = '';
  $self->{count} = 0;
  $self->{LIST} = undef;
  $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM $self->{table} WHERE id = '#FIELD=id#'";
  $self->{SELECTID} = undef;  #this is a hack to try to get key fields when using triggers/serials
  $self->{UPDATE} = undef;
  $self->{preINSERT} = undef;
  $self->{INSERT} = undef;
  $self->{DELETE} = undef;
  $self->{preDELETE} = undef;
  $self->{postDELETE} = undef;
  $self->{INSERTorder} = undef;  #insert order for db column names
  $self->{newOnly} = undef;
  $self->{db2data} = undef;  #map database columns to ->data hash, if not specified we assume same name in db and object
  $self->{data2db} = undef;
  $self->{data}->{main} = { };
  $self->{data}->{main}->{id} = { -Value => "", -Type => "hidden" } ;
  $self->{template}->{main} = "";
  $self->{profile}->{main} = { };
  $self->{javascript}->{main} = "";
  $self->{css}->{main} = "";
  $self->{imageDir} = $args{imageDir};

#   $self->{SELECT}[0] = "SELECT ##COLUMNS## FROM $self->{table} WHERE $self->{column} = #FIELD=id#";
#   $self->{INSERT}[0] = "INSERT INTO $self->{table} (##COLUMNS##) VALUES (##VALUES##)";
#   $self->{UPDATE}[0] = "UPDATE $self->{table} SET ##UPDATE## WHERE $self->{column} = #FIELD=id#";
#   $self->{DELETE}[0] = "DELETE FROM $self->{table} WHERE $self->{column} = #FIELD=id#";
  # find out ifis this being referenced from other tables before deleting

  if ($self->error)
  {
    $self->error('');
    return $self;
  }

  # instantiate anything unique to this module
  eval { $self->{portalVariables} = Portal::Data::Variables->new(langObj => $self->{langObj}); };
  if ($@)
  {
    $self->error($@);
    return $self;
  }
  if ($self->{portalVariables}->error)
  {
    $self->error($self->{portalVariables}->errorMessage);
    return $self;
  }

  # do validation
#   if (!$self->Portal::Accounting::Objects::Base::isValid)
#   {
#     # the error is set in the isValid() method.
#     return $self;
#   }

  # do anything else you might need to do.
  return $self;
}

=item bool isValid(void)

 Returns 0 for invalid or 1 for valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->error('');
    return 0;
  }

  #run over the pofile for main and check it against the parameters
  #what if I run over main looking for -DataType ?  no programmer may specify other fields that should not get populated but could get changed by othe programming.  use all fields
  if (exists $self->{profile}->{main})
  {
    my %fakeInput;
    $fakeInput{HTMLObjectFormInternalVariable} = 1;
    foreach my $field (keys %{$self->{data}->{main}})
    {
      $fakeInput{$field} = $self->get($field);
    }
#    my $result = $self->{formObj}->validate(input => \%fakeInput, profile => $self->{profile}->{main});
#    my $result = $self->validateForm(input => \%fakeInput);
my $result = 1;
    if ($self->{formObj}->error) { $self->error($self->{formObj}->errorMessage); }
    if (!$result)
    {
      #!!! set all parameters that are invalid?
      #we wouldn't have to bc the redisplay of the form would show the user what the problem is.
      $self->error($self->{formObj}->genErrorString('all'));
      return 0;
    }
  }
  # validate our parameters.
  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean isNew

  1 for new, 0 for not
  This routine should be implemented for your object

=cut

=item string dbRequired("db|data")

  This routine returns all of the data which is required by the select statements.
  optional:  db - to return the db column names, data - to return the data element names

=cut
sub dbRequired
{
  my $self = shift;
  my $basis = shift;
  if (! defined $basis)  {  $basis = "db";}
  else  {  $basis = "data";}
  my $return = "";
  #walk each select looking for #FIELD=#
  my @sqls = @{$self->{SELECT}};
  for (my $index = 0; $index < @sqls; $index++)
  {
    my $sql = $sqls[$index];
    while ($sql =~ /#FIELD=([^#]+)#/)
    {
      my $found = $1;
      my $map = $self->db2data(var => $found, index => $index);
      my $value = "foo";
      $sql =~ s/#FIELD=$found#/$value/g;
      if ($basis eq "db")  {  $return .= "$found:";  }
      else  {  $return .= "$map:";  }
    }
  }
  return $return;
}

=item (@sql, @plug) substitute(sql => "UPDATE", form => )

requires:  name of sql statements to substitute on
optional:  form - name of form to get data from
returns: aray of sql statements and plug values
summary:  This routine takes the sql statements specified from $self and
         runs substitutions on them to replace things like ##COLUMNS##
         the plug array that is returned is actually an array of arrayrefs
         to values that can be sent to dbi with plug => $plug[$index]

=cut
sub substitute
{
  my $self = shift;
  my %args = (form => "main", sql => "", @_);
  my $sqlarg = $args{sql};
  my $form = $args{form};
  if (! exists $self->{$sqlarg}) { $self->error("<p>Invalid arg to substitute: $sqlarg</p>\n"); return undef; }
  my @sqls = @{$self->{$sqlarg}};
  my @plugs;

  for (my $index = 0; $index < @sqls; $index++)
  {
    my $sql = $sqls[$index];
    my $update = "";
    my $columns = "";
    my $values = "";
    my $first = 1;
    my @plug;
    foreach my $key (@{$self->{INSERTorder}[$index]})
    {
      #build update, column, values
      if (!$first)
      {
        if (! exists $self->{newOnly}[$index]->{$key}) { $update .= ","; }
        $columns .= ",";
        $values .= ",";
      } else {  $first = 0; }
      $values .= " ? ";
      $columns .= " $key ";
      if (! exists $self->{newOnly}[$index]->{$key})
      {
        $update .= " $key = ? ";
      }
      #build up plug array
      if ( ($sqlarg eq "UPDATE" && ! exists $self->{newOnly}[$index]->{$key})
      || $sqlarg ne "UPDATE")
      {
        #map to data element in form
        my $map = $self->db2data(var => $key, index => $index);
        #special handling for checkboxes
        if ($self->{data}->{$form}->{$map}->{-Type} eq "checkbox")
        {
          if ($self->{data}->{$form}->{$map}->{checked} == 1)
          {
            if ($self->{data}->{$form}->{$map}->{-Boolean} )  { push @plug, 'true';  }
            elsif (defined $self->{data}->{$form}->{$map}->{-Value})
            {
              push @plug, $self->{data}->{$form}->{$map}->{-Value};
            }
            else { $self->error("Cannot use non boolean checkbox without a value set<br>\n"); return undef; }
          }
          else
          {
            if ($self->{data}->{$form}->{$map}->{-Boolean} )  { push @plug, 'false';  }
            elsif (defined $self->{data}->{$form}->{$map}->{-Value})
            {
              push @plug, 'NULL';
            }
            else { $self->error("Cannot use non boolean checkbox without a value set<br>\n"); return undef; }
          }
        }
        else
        {
#          if ($self->debug) { $self->debug("<p>f:$form<br>m:$map</p>\n");  }
          if (defined $self->{data}->{$form}->{$map}->{-Value})
          {
            push @plug, $self->{data}->{$form}->{$map}->{-Value};
          }
          else #may not exist or is just not set, since it is defined in insertorder use NULL!!! is that undef?
          {
            push @plug, undef;
          }
        }
      }
    }
    $sql =~ s/##UPDATE##/$update/g;
    $sql =~ s/##COLUMNS##/$columns/g;
    $sql =~ s/##VALUES##/$values/g;
    while ($sql =~ /#FIELD=([^#]+)#/)
    {
      my $found = $1;
      my $value = $self->{data}->{$form}->{$found}->{-Value};
      $sql =~ s/#FIELD=$found#/$value/g;
    }
    #the trick is to send back the correct amount of plug vars.
    if ($sql =~ m/\?/)
    {
      $plugs[$index] = \@plug;
    }
    else
    {
      my @p;
      $plugs[$index] = \@p;
    }
    $sqls[$index] = $sql;
    if ($self->debug) { $self->debug("<p>$index:$sql<br>".join(",",@plug)."</p>\n"); }
  }
  return (\@sqls, \@plugs);
}

=item integer select()

  Use:  $obj->populate(id => $foo, other => $bar);
        $obj->select()

  This routine uses the $self->{INSERTorder} to automatically load the data
  from the sql call into the current object's ->{data} hash
  required:  must be popluated with critical data before calling select on the object
      To find out which data is critical call $obj->dbRequired
  optional:
  returns: undef on error, # of records selected for success

=cut
sub select
{
  my $self = shift;
#   if ($self->debug) { $self->debug("<p>Made it to select!</p>\n"); }
  my $records = 0;
  if (! defined $self->{SELECT} || ! defined $self->{SELECT}[0] || $self->{SELECT}[0] eq "") { $self->error("Must define sql select statement<br>\n"); return undef;  }
  #subst on sql in SELECT ARRAYS
  my ($sqls, $plugs) = $self->substitute(sql => "SELECT");
  for (my $index = 0; $index < @{$sqls}; $index++)
  {
    my $sql = $sqls->[$index];
    if ($self->debug) {  $self->debug($sql);  }
    my $result = $self->{dbHandle}->read($sql);
    if ($self->{dbHandle}->error)
    {
      $self->error("<br>".$self->{dbHandle}->errorMessage."<br>\n\n$sql");
      return undef;
    }
    if (my $DB = $result->fetchrow_hashref)
    {
      foreach my $form (keys %{$self->{data}})
      {
        foreach my $key (keys %{$DB})
        {
          my $mapKey = $self->db2data(var => $key, index => $index);
          if (defined $self->{data}->{$form}->{$mapKey})
          {
            if ($self->{data}->{$form}->{$mapKey}->{-Type} eq "checkbox")
            {
              #data could be NULL/something or boolean TRUE/FALSE
              if ($DB->{$key} eq "t" || $DB->{$key} eq "true" || $DB->{$key} == 1)
              {
                $self->{data}->{$form}->{$mapKey}->{checked} = 1;
              }
              elsif ($DB->{$key} eq "f" || $DB->{$key} eq "false" || $DB->{$key} == 0)
              {
                $self->{data}->{$form}->{$mapKey}->{checked} = 0;
              }
              else #NULL OR empty or something non boolean
              {
                if ($DB->{$key} eq "") { $self->{data}->{$form}->{$mapKey}->{checked} = 0;  }
                else { $self->{data}->{$form}->{$mapKey}->{checked} = 1;  }
              }
            }
            else
            {
              $self->{data}->{$form}->{$mapKey}->{-Value} = $DB->{$key};
            }
            #this check may not need to be here bc the routines that do output are chekcing it
  #           if (exists $self->{newOnly}[$index]->{$key})
  #           {
  #             $self->{data}->{$form}->{$mapKey}->{-ReadOnly} = 1;
  #           }
          }
        }
      }
      $records++;
    }
  }
  return $records;
}

=item $array selectId()

  DEPRECATED:  try to use insertId instead

  Use:  $obj->populate(foo => $foo, other => $bar);
        $obj->selectId()

  This routine uses the data in the object to select keys from the key field.
  Probably the data just entered with a trigger on the key field produced an
  unknown value in the key field.  That same data might lead to multiple keys
  when queried.
  required:  must be popluated with critical data before calling select on the object
  returns: undef on error, $array from db records

=cut
sub selectId
{
  my $self = shift;
  if ($self->debug) { $self->debug("<p>Made it to selectId!</p>\n"); }
  if (! defined $self->{SELECTID} || ! defined $self->{SELECTID}[0] || $self->{SELECTID}[0] eq "") { return (); }
  my ($sqls, $plugs) = $self->substitute(sql => "SELECTID");
  for (my $index = 0; $index < @{$sqls}; $index++)
  {
    my $sql = $sqls->[$index];
    if ($self->debug) {  $self->debug($sql);  }
    my $result = $self->{dbHandle}->read($sql);
    if ($self->{dbHandle}->error)
    {
      $self->error("<br>".$self->{dbHandle}->errorMessage."<br>\n\n$sql");
      return undef;
    }
    if (my $DB = $result->fetchrow_arrayref)
    {
      return $DB;
    }
  }
  return ();
}

sub insertId
{
  my $self = shift;
  return $self->{insert_id};
}

=item integer write(form)

  This routine inserts or updates the data in the
  given form in the database.  It knows which routine
  to call by checking the new flag.

=cut
sub write
{
  my $self = shift;
  if ($self->isNew(@_))
  {
    return $self->insert(@_);
  }
  return $self->update(@_);
}

=item integer insert(commit =>, form =>)

  Routine that tries to take data in object and
  inserts it into the database from the main data hash
  optional: string - name of a different data hash to take data from
  returns: undef for error, number of records on success

  how do I know what the id of the object is that was just inserted?
  I can maybe use the $self->{id} field and look for a nextval?

=cut
sub insert
{
  my $self = shift;
  my %args = (commit => 1, @_);
  my $form = shift;
  if (!$self->isValid) { return -1; }
  my $records = 0;
  if (not defined $form) { $form = "main";  }
  if (! defined $self->{INSERT} || ! defined $self->{INSERT}[0] || $self->{INSERT}[0] eq "") { $self->error("<div>Must define sql INSERT statement</div>"); return undef;  }
  #subst on sql in INSERT ARRAYS
  my ($sqls, $plugs) = $self->substitute(sql => "INSERT");
  for (my $index = 0; $index < @{$sqls}; $index++)
  {
    my $sql = $sqls->[$index];
    my $plug = $plugs->[$index];
    my $result = $self->{dbHandle}->write(sql => $sql, plug => $plug);
    if (defined $self->{table} && defined $self->{id})
    {
      $self->{insert_id} = $self->{dbHandle}->getID($self->{table}.".".$self->data2db($self->{id}));
      if (defined $self->{data}->{$self->{id}})
      {
        $self->set($self->{id}, $self->{insert_id});
      }
    }
    if ($self->{dbHandle}->error)
    {
      $self->invalid('database',$sql.join(",",@{$plug}),$self->{dbHandle}->errorMessage);
      $self->error($self->{dbHandle}->errorMessage."$sql:".join(",",@{$plug}));
      return -1;
    }
    $records += $result;
  }
  if ($args{commit})
  {
    if (!$self->error) { $self->{dbHandle}->commit; }
    else { $self->{dbHandle}->rollback; }
  }
  return $records;
}

=item integer update("string")

  Routine that tries to take data from internal object and
  update the database
  optional:  name of form to use in data hash
  returns: undef for error, -1 for cannot update, or number of records updated on success

=cut
sub update
{
  my $self = shift;
  my %args = (commit => 1, @_);
  if (!$self->isValid) { return -1; }
  my $form = shift;
  my $records = 0;
  if (! defined $form) { $form = "main";  }
  if (! defined $self->{UPDATE} || ! defined $self->{UPDATE}[0] || $self->{UPDATE}[0] eq "") { $self->error("<div>Must define sql UPDATE statement</div>"); return undef;  }
  #subst on sql in UPDATE ARRAYS
  my ($sqls, $plugs) = $self->substitute(sql => "UPDATE", form => $form);
  if ($self->error) { $self->error(" "); return undef;  }
  for (my $index = 0; $index < scalar @{$sqls}; $index++)
  {
    my $sql = $sqls->[$index];
    my $plug = $plugs->[$index];
    if ($self->debug) { $self->debug("<div>$sql:". join(",", @{$plug}) ."</div>");  }
    my $result = $self->{dbHandle}->write(sql => $sql, plug => $plug);
    if ($self->{dbHandle}->error)
    {
      $self->invalid('database','error',$self->{dbHandle}->errorMessage."<div>$sql</div>");
      return -1;
    }
    $records += $result;
  }
  $self->{dbHandle}->commit;
  return $records;
}

sub printForm
{
  my $self = shift;
  my %args = (form => 'main', @_);
  my $form = $args{form};
  my $s;
  if ($self->isEntryInvalid('database'))
  {
    $s .= $self->genErrorField('invalid','database');
  }
  if (!$self->isNew && defined $self->{newOnly})
  {
    for (my $index = 0; $index < @{$self->{newOnly}}; $index++)
    {
      foreach my $field (keys %{$self->{newOnly}[$index]})
      {
        my $map = $self->db2data($field);
        if (defined $self->{data}->{$form}->{$map})
        {
          $self->{data}->{$form}->{$map}->{-ReadOnly} = 1;
        }
      }
    }
  }
  my %form = $self->SUPER::printForm(@_);
  if ($self->{formObj}->error) { $s .= $self->{formObj}->errorMessage; }
  elsif (defined $self->{css}->{$args{form}})
  {
    $form{css} .= $self->{css}->{$args{form}};
  }
  $form{body} .= $s;
  return %form;
}

=item integer delete()

  This routine will attempt to delete the record for the object internal id.
  optional:  form - name of the data hash to use.  defaults to main
  returns: undef on error, integer on success,
           also sets internal variable $self->result
    integer: -1 if preDELETE finds a dependency (in which case delete is aborted)
             or number of records if DELETE sql stmt is run
    $self->result:  the DB record found if delete is aborted,
             or the result of postDELETE if delete was successful

=cut
sub delete
{
  my $self = shift;
  my %args = (commit => 1, @_);
  my $form = $args{form};
  my $r = 0;
  if (! defined $form) { $form = "main"; }
  if (! defined $self->{DELETE} || ! defined $self->{DELETE}[0] || $self->{DELETE}[0] eq "") { $self->error("Must define sql DELETE statement<br>\n"); return undef;  }
  #subst on sql in DELETE ARRAYS
  my $check = $self->preDELETE($form);
  my $post = $self->postDELETE($form);
  if (! defined $check) { $self->error("Error in delete when calling preDELETE"); return undef; }
  if (! defined $post) { $self->error("Error in delete when calling postDELETE"); return undef; }
  if ($check ne "")
  {
    $self->{result} = $check;
    return -1;
  }
  $self->{result} = $post;
  my ($sqls, $plugs) = $self->substitute(sql => "DELETE");
  for (my $index = 0; $index < scalar @{$sqls}; $index++)
  {
    my $sql = $sqls->[$index];
    my $plug = $plugs->[$index];
    my $result = $self->{dbHandle}->write(sql => $sql, plug => $plug);
    if ($self->{dbHandle}->error)
    {
      $self->error("<br>".$self->{dbHandle}->errorMessage."<br>\n\n$sql");
      $self->{dbHandle}->rollback;
      return undef;
    }
    $r += $result;
    if ($self->debug) { $self->debug("<br>rs:$result:$r<br>$sql<br>\n"); }
  }

  if ($args{commit})
  {
    if (!$self->error) { $self->{dbHandle}->commit; }
    else { $self->{dbHandle}->rollback; }
  }
  return $r;
}

#retuns undef on error
# or value from sql statement for post delete
sub postDELETE
{
  my $self = shift;
  my $form = shift;
  my $return = "";
  if (! defined $form) { $form = "main"; }
  if (! defined $self->{postDELETE} || ! defined $self->{postDELETE}[0] || $self->{postDELETE}[0] eq "") { return "";  }
  #subst on sql in postDELETE ARRAYS
  my ($sqls, $plugs) = $self->substitute(sql => "postDELETE");
  for (my $index = 0; $index < @{$sqls}; $index++)
  {
    my $sql = $sqls->[$index];
    my $plug = $plugs->[$index];
    if (!defined $sql || $sql eq "") { next; }
    #run the check code
    if ($self->debug) { $self->debug("<br".$sql."<br>");  }
    my $result = $self->{dbHandle}->read(sql => $sql, plug => $plug);
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{dbHandle}->errorMessage."<br>\n\n$sql<br>");
      return undef;
    }
    #if any rows are returned then return it
    while (my @data = $result->fetchrow_array)
    {
      $return .= $data[0]."\n";
    }
  }
  return $return;
}

#returns undef on error
# or empty if nothing found
# or record found
sub preINSERT
{
  my $self = shift;
  my $form = shift;
  my $return = "";
  if (! defined $form) { $form = "main"; }
  if (! defined $self->{"preINSERT"} || ! defined $self->{"preINSERT"}[0]) { return "";  }
  #subst on sql in preINSERT ARRAYS
  my ($sqls, $plugs) = $self->substitute(sql => "preINSERT");
  for (my $index = 0; $index < @{$sqls}; $index++)
  {
    my $sql = $sqls->[$index];
    return $sql;
    my $plug = $plugs->[$index];
    #run the check code
    if ($self->debug) { $self->debug("<p>$sql<br>id: $self->{data}->{$form}->{$self->{id}}->{-Value}<br>".join(",",@{$plug})."</p>");  }
    my $result = $self->{dbHandle}->read(sql => $sql, plug => $plug);
    if ($self->{dbHandle}->error)
    {
      $self->error("<p>$self->{dbHandle}->errorMessage<br>\n\n$sql</p>");
      return undef;
    }
    #if any rows are returned then return it
    while (my @data = $result->fetchrow_array)
    {
      $return .= $data[0];
    }
  }
  return $return;
}

sub preDELETE
{
  my $self = shift;
  my $command = "DELETE";
  my $form = shift;
  my $return = "";
  if (! defined $form) { $form = "main"; }
  if (! defined $self->{"pre$command"} || ! defined $self->{"pre$command"}[0]) { return "";  }
  #subst on sql in preINSERT ARRAYS
  my ($sqls, $plugs) = $self->substitute(sql => "pre$command");
  for (my $index = 0; $index < @{$sqls}; $index++)
  {
    my $sql = $sqls->[$index];
    my $plug = $plugs->[$index];
    #run the check code
    if ($self->debug) { $self->debug("<p>$sql<br>id: $self->{data}->{$form}->{$self->{id}}->{-Value}<br>".join(",",@{$plug})."</p>");  }
    my $result = $self->{dbHandle}->read(sql => $sql, plug => $plug);
    if ($self->{dbHandle}->error)
    {
      $self->error($self->{dbHandle}->errorMessage."<br>\n\n$sql<br>");
      return undef;
    }
    #if any rows are returned then return it
    while (my @data = $result->fetchrow_array)
    {
      $return .= $data[0]."\n";
    }
  }
  return $return;
}

=item string db2data("var") or db2data(var =>, index =>)

summary:  Routine looks in db2data hash for the given database column name.  If found
  it returns the corresponding data element name.  If not found the data mapping is
  assumed to be the same as the given name and that is returned.
requires:  if only 1 argument it is the variable to map.  Will assume the index to be 0
optional:  index - corresponds to the arrays in the internal data structures
returns:  The corresponding data element name for the given database column name.

=cut
sub db2data
{
  my $self = shift;
  my $map;
  my $index = 0;
  if (@_ == 1)  {  $map = shift;  }
  else
  {
    my %args = (index => $index, @_);
    $map = $args{var};
    $index = $args{index};
  }
  if (defined $self->{db2data}[$index]->{$map}) { return $self->{db2data}[$index]->{$map};  }
  return $map;
}

=item string data2db("var") or data2db(var =>, index =>)

summary:  Routine looks in data2db hash for the given data element name.  If found
  it returns the corresponding database column name.  If not found the data mapping
  is assumed to be the same as the given name and that is returned.
requires:  if only 1 argument it is the variable to map.  Will assume the index to be 0
optional:  index - corresponds to the arrays in the internal data structures
returns:  The corresponding database column name for the given data element.

=cut
sub data2db
{
  my $self = shift;
  my $map;
  my $index = 0;
  if (@_ == 1)  {  $map = shift;  }
  else
  {
    my %args = (index => $index, @_);
    $map = $args{var};
    $index = $args{index};
  }
  if (defined $self->{data2db}[$index]->{$map}) { return $self->{data2db}[$index]->{$map};  }
  return $map;
}

=item boolean warning("string") or boolean warning()

  similar to the error routine this routine will set a warning
  statement and turn on warning if called.  access throught
  $obj->warningString

=cut
sub warning
{
  my $self = shift;
  my @callerArgs = caller(1);
  (my $subName = $callerArgs[3]) =~ s/^(.+)(::)([^:]+)$/$1->$3/;
  my $callerErrStr = "";

  if (scalar @_ > 0)
  {
    # we are setting an warning condition.
    if (scalar @_ == 1)
    {
      $self->{warningString} .= $callerErrStr . @_[0]."<br>";
    }
    else
    {
      # handle the sprintf case.
      if (@_[0] =~ /(?<!\\)%[^%]/)
      {
        # build up the string to eval for the sprintf.
        my $str = "\"@_[0]\"";
        for (my $i=1; $i < scalar @_; $i++)
        {
          $str .= ", \"@_[$i]\"";
        }
        $self->{warningString} .= $callerErrStr;
        eval "\$self->{warningString} .= sprintf($str);";
        if ($@)
        {
          $self->warning($callerErrStr . $@);
          return;
        }
      }
      else
      {
        $self->{warningString} .= $callerErrStr . join(" ", @_);
      }
    }
    $self->{warningString} .= "\n";
    $self->{warning} = 1;
  }

  return $self->{warning};
}

=item boolean debug("string") or boolean debug()

  similar to the error routine this routine will set a debug
  statement and turn on debug if called.  access throught
  $obj->debugString

=cut
sub debug
{
  my $self = shift;
  my @callerArgs = caller(1);
  (my $subName = $callerArgs[3]) =~ s/^(.+)(::)([^:]+)$/$1->$3/;
  my $callerErrStr = "$subName";

  if (scalar @_ > 0)
  {
    # we are setting an error condition.
    if (scalar @_ == 1)
    {
      $self->{debugString} .= $callerErrStr . @_[0]."<br>";
    }
    else
    {
      if (@_[0] eq "debugString")
      {
        my %args = ( @_ );
        if (!exists $args{debugString})  # make sure we get the debugString argument!
        {
          $self->debug($callerErrStr . "<b>debugString</b> is missing!<br>\n");
          return;
        }
        else
        {
          $self->{debugString} .= $callerErrStr . $args{debugString};
        }
      }
      else
      {
        # handle the sprintf case.
        if (@_[0] =~ /(?<!\\)%[^%]/)
        {
          # build up the string to eval for the sprintf.
          my $str = "\"@_[0]\"";
          for (my $i=1; $i < scalar @_; $i++)
          {
            $str .= ", \"@_[$i]\"";
          }
          $self->{debugString} .= $callerErrStr;
          eval "\$self->{debugString} .= sprintf($str);";
          if ($@)
          {
            $self->debug($callerErrStr . $@);
            return;
          }
        }
        else
        {
          $self->{debugString} .= $callerErrStr . join(" ", @_);
        }
      }
    }
    $self->{debugString} .= "\n";
    $self->{debug} = 1;
  }

  return $self->{debug};
}

sub resetDebug
{
  my $self = shift;
  $self->{debugString} = "";
}

=item @array selectArray

  This routine uses $self->{LIST} sql statement to create an instance of this object for each record returned.
  returns an array or undef on error.

=cut
sub selectArray
{
  my $self = shift;
  return $self->selectArrayHash("array", @_);
}

=item @array selectHash

  This routine uses $self->{LIST} sql statement to create an instance of this object for each record returned.
  returns an hash with the $self->id field as the key or undef on error.

=cut
sub selectHash
{
  my $self = shift;
  return $self->selectArrayHash("hash", @_);
}

sub selectArrayHash
{
  my $self = shift;
  my $which = shift;
  my $object = ref($self);
  if (defined $self->{object} && $self->{object} ne '') { $object = $self->{object}; eval("use $object;"); }
  if ($which ne "array" && $which ne "hash") { $self->error("<p>Invalid option give to selectArrayHash:$which</p>\n"); return undef; }
  my $return;
  if (! defined $self->{LIST}) { $self->error("<p>LIST sql not defined but subroutine is trying to use it</p>\n"); return undef; }
  my ($sqls, $plugs) = $self->substitute(sql => "LIST");

  my $result = $self->{dbHandle}->read($sqls->[0]);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef;  }
  while (my @d = $result->fetchrow_array)
  {
    my $o;
    my $e = '$o = '. $object .'->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle}, @_);' ;
    eval($e);
    if (!defined $o) { $self->error("<p>Could not create object needed of type: ".$object."<br>$e</p>\n");  return undef;  }
    if ($o->error) { $self->error("<p>Could not create object needed of type: ".$object."<br>$e<br>".$o->errorMessage."</p>\n");  return undef;  }
    $o->set($o->{id}, $d[1]);
    if (! $o->error)  {  $o->select;  }
    if ($o->error) { $self->error($o->errorMessage);  return undef; }
    if ($which eq "hash")
    {
      $return->{$d[1]} = $o;
    }
    else
    {
      push @{$return}, $o;
    }
  }
  if ($which eq "hash")
  {
    if (not defined $return) { return (); }
    return %{$return};
  }
  if (not defined $return) { return (); }
  return @{$return};
}

=item HTMLObject::Form::Options selectOptions

  This routine uses the internal $self->{LIST} sql to generate a list of name /value pairs for this type of object.
  Then that list is turned into the format needed for the HTMLObject::Form Options hash.

=cut
sub selectOptions
{
  my $self = shift;
  return $self->{formObj}->createSelectOptions($self->selectSTH);
}

=item dbHandle selectDBH()

  This routine uses the internal $self->{LIST} sql to generate a list of name /value pairs for this type of object.

=cut
sub selectSTH
{
  my $self = shift;
  if (! defined $self->{LIST}) { $self->error("<p>LIST sql not defined but subroutine is trying to use it</p>\n"); return undef; }
  my ($sqls, $plugs) = $self->substitute(sql => "LIST");
  my $result = $self->{dbHandle}->read(sql => $sqls->[0], plug => $plugs->[0]);
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return undef;  }
  return $result;
}

sub getDateArray
{
  return Today();
}

sub getCurrentDate
{
  my $self = shift;
  my @date = $self->getDateArray;
  return $date[0]."-".sprintf("%02d", $date[1])."-".sprintf("%02d", $date[2]);
}

sub printList
{
  my $self = shift;
  my $return;
  my @data = $self->selectArray(@_);
  if ($self->error) { return undef; }
  if (@data > 0)
  {
    $return .= "<table class=\"primary\">\n";
    my $first = 1;
    foreach my $d (@data)
    {
      if ($first)
      {
        $first = 0;
        $return .= $d->printHTMLHeader;
      }
      $return .= $d->printHTMLRow;
      if ($self->debug)  {  $self->debug("<p>rowData: $d->{data}->{main}->{$d->{id}}->{-Value}</p>\n");  }
    }
    $return .= "</table>\n";
  }
  return $return;
}

sub printNew
{
  my $self = shift;
  my $New = $self->{langObj}->map("New");
  my $app = $self->{data}->{main}->{app}->{-Value};
  my $state = $self->{data}->{main}->{state}->{-Value};
  my $l = $self->{data}->{main}->{ledgerCode}->{-Value};
  my $return = <<"ENDOF";
<form name="new">
  <input type="submit" name="submit" value="$New">
  <input type="hidden" name="app" value="$app">
  <input type="hidden" name="state" value="$state">
  <input type="hidden" name="command" value="new">
  <input type="hidden" name="ledgerCode" value="$l">
</form>
ENDOF
  return $return;
}

sub populate
{
  my $self = shift;
  my $foo = $self->SUPER::populate(@_);
  #this code will not stop the user from sending a differnet value which might show up if this object is printed
  #but the sql code will not USE the value bc it checks the newOnly flag
  if (!$self->isNew && defined $self->{newOnly})
  {
    foreach my $var (keys %{$self->{newOnly}[0]})
    {
       my $map = $self->db2data($var);
       if (exists $self->{data}->{main}->{$map})
       {
         $self->{data}->{main}->{$map}->{-ReadOnly} = 1;
       }
       else
       {
         $self->error('<p>$self->{data}->{main}->{'.$map.'} does not exist!</p>');
       }
    }
  }
  return $foo;
}

=item %formHash pagerForm(form => 'main', count => $self->{count}, number => 10, page => 'page', )

  this routine will create a paging form with links for going forward and
  backward when there are more entries available than the number displayable on a page.
  To use it you must define a form that would normally be used to return a data set.
  One example might be a search form.  The form must contain a page parameter so that
  it can be changed by the output from this form.

  returns:  hash with body, head, etc. that can be put into HTMLobject
  requires:  count - the total number of entries in the data set. can be overridden but
        defaults to the internal variable $self->{count}
  optional:
      number - the number of entries per page.  defaults to 0 which means only output the
        count of the transactions but no paging form
      page - the element on the form that holds the current page number or the next page
        desired when submitting to the back end.  defaults to 'page'.
      form - the form that holds the data to use on the paging form.  All data will be hidden.
        defaults to 'pager'.

=cut
sub pagerForm
{
  my $self = shift;
  my %args = (form => 'pager', count => $self->{count}, number => 0, page => 'page', @_);
  my $count = $args{count};
  my $num = $args{number};
  my $form = $args{form};
  my $page = $args{page};
  if (! defined $self->{data}->{$form}) { $self->error("<p>No data '$form' defined during call to pagerForm()!</p>\n"); return undef; }
  if ($num !~ /\d+/) { $self->error("<p>Invalid number '".$num."'!</p>\n"); return undef; }
  my $total = "$count ".$self->{langObj}->map("Transactions")." ".$self->{langObj}->map("Total");
  my $Refresh = $self->{langObj}->map('Refresh');
  my %form = $self->printForm(form => $form, @_);
  if ($num == 0)
  {
    $form{body} .= "<div class='pager printHide'><a onclick=\"document.$form.$page.value='0'; document.$form.submit();\" title=\"$Refresh\">$total</a></div>\n" ;
    return %form;
  }
  if ($self->error) { return %form; }
  my $of = $self->{langObj}->map('of');
  my $Page = $self->{langObj}->map('Page');
  my $pages = sprintf("%.0f", $count / $num + .5);
  my $pageN = $self->{data}->{$form}->{$page}->{-Value};
  $form{body} .= "\n\n<div class='pager printHide'>";
  if ($count <= $num)  #page always == 1
  {
    $form{body} .= " $Page <a onclick=\"document.$form.$page.value='$pageN'; document.$form.submit();\" title=\"$Refresh\">$pageN</a> $of $pages \n";
    $form{body} .= "\n";
  }
  elsif ($count <= $num * 2) #only two pages
  {
    if ($pageN == 1)
    {
      $form{body} .= " $Page <a onclick=\"document.$form.$page.value='$pageN'; document.$form.submit();\" title=\"$Refresh\">$pageN</a> $of $pages \n";
      $form{body} .= " | <a onclick=\"document.$form.$page.value='2'; document.$form.submit(); \">&gt;&gt;</a> \n";
    }
    else
    {
      $form{body} .= " <a onclick=\"document.$form.$page.value='1'; document.$form.submit(); \">&lt;&lt;</a> |\n";
      $form{body} .= " $Page <a onclick=\"document.$form.$page.value='$pageN'; document.$form.submit();\" title=\"$Refresh\">$pageN</a> $of $pages \n";
    }
    $form{body} .= "\n";
  }
  elsif ($count <= $num * $pageN) # on last page with at least three
  {
    $form{body} .= " <a onclick=\"document.$form.$page.value='1'; document.$form.submit(); \">&lt;&lt;</a>\n";
    $form{body} .= " | <a onclick=\"javascript: document.$form.$page.value='" . ($pageN - 1) ."'; document.$form.submit(); \">&lt;</a> |\n";
    $form{body} .= " $Page <a onclick=\"document.$form.$page.value='$pageN'; document.$form.submit();\" title=\"$Refresh\">$pageN</a> $of $pages \n";
    $form{body} .= "\n";
  }
  else #3 or more pages but not on last one
  {
    if ($pageN > 1)
    {
      $form{body} .= " <a onclick=\"document.$form.$page.value='1'; document.$form.submit(); return false; \">&lt;&lt;</a>\n";
      $form{body} .= " | <a onclick=\"javascript: document.$form.$page.value='" . ($pageN - 1) ."'; document.$form.submit(); \">&lt;</a> | \n";
    }
    $form{body} .= " $Page <a onclick=\"document.$form.$page.value='$pageN'; document.$form.submit();\" title=\"$Refresh\">$pageN</a> $of $pages \n";
    $form{body} .= " | <a onclick=\"javascript: document.$form.$page.value='" . ($pageN + 1) ."'; document.$form.submit(); \">&gt;</a>\n";
    $form{body} .= " | <a onclick=\"javascript: document.$form.$page.value='$pages'; document.$form.submit(); \">&gt;&gt;</a>\n";
    $form{body} .= "\n";
  }
  $form{body} .= "($total)</div>\n";
  return %form;
}

=item validateForm

  had to do this routin to set the new field if there is one.  otherwise stuff doesn't work because populate is NOT called.

=cut
# sub validateForm
# {
#   my $self = shift;
#   my $f = $self->SUPER::validateForm(@_);
#   if (defined $self->{data}->{main}->{new})
#   {
#     $self->set('new', $self->{formObj}->{data}->{new}); #!!!
#   }
#   return $f;
# }

sub today
{
  my $self = shift;
  return $self->getCurrentDate(@_);
}

=item commify

 requires: value
 optional: US => 0|1 - defaults to 1
 returns: value with thousands seperators
 This routine commifies the specified value according to standards.
 An alternative of EU standards would not be hard.

 !!!CURRENTLY BROKEN WHY TRYING TO DO NON US version

=cut
sub commify
{
  my $self = shift;
  my $seperator = ',';
  my $decimal = '\.';
  my $value;
  if (scalar @_ > 1)
  {
    my %args = ( value => "0", US => 1, @_ );
    if (!$args{US})
    {
      $seperator = '.';
      $decimal = ',';
    }
    $value = reverse $args{value};
  }
  else
  {
    $value = reverse @_[0];
  }
#  $value =~ s/(\d\d\d)(?=\d)(?!\d*\.)/$1,/g;
#  $value =~ s/(\d\d\d)(?=\d)(?!\d*\,)/$1./g;
  eval('$value =~ s/(\d\d\d)(?=\d)(?!\d*'.$decimal.')/$1'.$seperator.'/g;');
#  print '$value =~ s/(\d\d\d)(?=\d)(?!\d*'.$decimal.')/$1'.$seperator.'/g;'."\n";

  $value = reverse $value;
  return $value;
}

sub log
{
  my $self = shift;
  my $text = shift;
  my $sql = "INSERT INTO log_tb (text) VALUES (?)";
  $self->{dbHandle}->write(sql => $sql, plug => [$text]);
  if ($self->{dbHandle}->error)
  {
    $self->error($self->dbHandle->errorMessage);
    return 0;
  }
  return 1;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)


=head1 SEE ALSO

perl(1), Portal::Base(3)

=cut
