# Trial.pm - The Object Class that provides a Trial Object
#$Id: Trial.pm,v 1.1.2.3 2007/03/28 04:29:46 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2003 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Reports::Objects::Trial;
use strict;
use Portal::Accounting::Objects::BaseL;
use Date::Calc qw(Today);
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Objects::BaseL Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.1.2.3 $';

=head1 NAME

Trial - Object used to build a Trial Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Reports::Objects::Trial;
  my $obj = Portal::Accounting::Reports::Objects::Trial->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  print $obj->printForm;

  To validate data from user:

  use Portal::Accounting::Reports::Objects::Trial;
  my $obj = Portal::Accounting::Reports::Objects::Trial->new;
  if ($obj->error())
  {
    die $obj->errorMessage();
  }
  if ($obj->validateForm(form => 'main', input => \%input))
  {
    if ($obj->select)
    {
      if ($obj->error)
      {
        print ($obj->errorMessage);
      }
      else
      {
        print $obj->printReport;
      }
    }
  }

=head1 DESCRIPTION

Trial holds forms for dealing with Federal 941 payroll reports.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new( )

    Creates a new instance of Portal::Accounting::Objects::Trial.
    See Portal::Accounting::Objects::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( ledgerCode => '', dbHandle => undef, @_ );

  if ($self->error)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{dbHandle} = $args{dbHandle};
  my @date = Today();
  my $l = $self->{langObj};
  $self->{defaultForm} = "main";
  $self->{data}->{main} = {
    "app" => { -Type => "hidden", -Value => "Accounting" },
    "state" => { -Type => "hidden", -Value => "Reports::Trial" },
    "command" => { -Type => "hidden", -Value => "report" },
    "report" => { -Type => "hidden", -Value => "Trial" },
    "ledgerCode" => { -Type => "hidden", -Value => $args{ledgerCode},},
    "from" =>{ -Type => 'date-picker', -year => $date[0], -DataType => 'string', -Label => $l->map('From'), },
    "to" => { -Type => 'date-picker', -year => $date[0], -DataType => 'string',  -Label => $l->map('To'), },
    "type" => { -Type => "radio", -DataType => 'string', -Value => 'range',
      -Options => { names => [$l->map("Range"), $l->map("Balance")], values => [qw(range balance)] },
      onclick => "return toggle(this.value);",
    },
  };
   $self->{css}->{report} = <<"ENDOF";
table { width: 100%; }
.debit,.credit {
  text-align: right;
}
.label {
  text-align: left;
}
.code {
  text-align: left;
}
.account_type {
  text-align: left;
}
.account_num {
  display: none;
}
.account_type_num {
  display: none;
}
.level {
  display: none;
}
ENDOF
   $self->{javascript}->{main} = <<"ENDOF";
function toggle(what)
{
  if (what == 'balance')
  {
    hide('fromDiv');
  }
  else
  {
    showDiv('fromDiv');
  }
  return true;
}

function init()
{
 //stub
}

ENDOF
   $self->{template}->{report} = <<"ENDOF";
<form id="drill" method="post" target="drill">
  <input type="hidden" name="app" id="app" value="Accounting" />
  <input type="hidden" name="state" id="state" value="Reports::Detailed" />
  <input type="hidden" name="command" id="command" value="run_report" />
  <input type="hidden" name="ledgerCode" id="ledgerCode" value="$args{ledgerCode}" />
  <input type="hidden" name="sql_statement" id="sql_statement" value="" />
  <input type="hidden" name="id" value="1" />
  <input type="hidden" name="date" value="1" />
  <input type="hidden" name="payee_payer" value="1" />
  <input type="hidden" name="subEntity" value="1" />
  <input type="hidden" name="account_name" value="1" />
  <input type="hidden" name="sub_account_name" value="1" />
  <input type="hidden" name="debit" value="1" />
  <input type="hidden" name="credit" value="1" />
</form>
ENDOF
   $self->{template}->{main} = <<"ENDOF";
<form #FORMARGS# class="primary" style="width: 50%; text-align: right;">
  #FORMERRORSTRING#
  #FIELD=app#
  #FIELD=state#
  #FIELD=command#
  #FIELD=ledgerCode#
  <div style="float: left; padding: 1em; text-align: left;">#FIELD=type#</div>
  <div style="padding: 1em;">
    <div id="fromDiv">
      #LABEL=from# #FI=from#
    </div>
    <div>
      #LABEL=to# #FI=to#
    </div>
  </div>
  <input style="clear: both; " type="submit" name="submit" value="#LANG=Submit#" />
</form>
ENDOF
  $self->{profile}->{main} = {
    required => [ qw( to type ) ],
    optional => [ qw( from ) ],
    constraints => {
      to => qr/^\d{4}-\d{1,2}-\d{1,2}$/,
      from => qr/^\d{4}-\d{1,2}-\d{1,2}$/,
      type => qr/^balance|range$/,
    },
  };

  # do anything else you might need to do.

  return $self;
}

sub isNew
{
  return 1;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  if ($self->get('type') eq 'range') { push(@{$self->{profile}->{main}->{required}}, 'from'); }

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.
  if ($self->get('type') eq 'range' && $self->get('from') eq '')
  {
    $self->invalid('from','','From is required with date range.');
  }

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
#    $self->error($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item boolean select

  Gathers data from database for this report.

=cut
sub select
{
  my $self = shift;
  my $DATERANGE;
  my $p = $self->langObj;
  my $from = $self->get('from');
  my $to = $self->get('to');
  my $type = $self->get('type');
  my $l = $self->get('ledgerCode');
  my @plug = ( $l , $to, $l, $to );
  my $DATES = '<div style="text-align: center; font-weight: bold;">';
  if ($type eq 'range')
  {
    $DATES .= "$from - ";
    $DATERANGE = " AND d.trans_date >= ? ";
    @plug = ( $l , $to, $from, $l, $to, $from );
  }
  $DATES .= <<"ENDOF";
$to
    <input type="hidden" id="to" value="$to" />
    <input type="hidden" id="from" value="$from" />
    <input type="hidden" id="type" value="$type" />
  </div>
ENDOF
  $self->{DATA} .= $DATES;
  $self->{headers} = {
    level => 'level',
    label => $p->map("Account"),
    code => $p->map("Account Code"),
    account_num => "account_num",
    account_type_num => "account_type_num",
    account_type => $p->map("Account Type"),
    debit => $p->map("Debit"),
    credit => $p->map("Credit"),
  };
  $self->{LIST}[0] = <<"ENDOF";
SELECT
  1 AS level,
  a.name AS label,
  a.code AS code,
  d.account_num,
  a.type AS account_type_num,
  t.name AS account_type,
  SUM(
    CASE WHEN d.dorc = true THEN
      d.amount
    ELSE 0.00
    END
  ) AS debit,
  SUM(
    CASE WHEN d.dorc = false THEN
      d.amount
    ELSE 0.00
    END
  ) AS credit
 FROM debitcredit_tb AS d, account_tb AS a, acct_type_tb AS t
 WHERE
  d.account_num = a.number
  AND a.type = t.num
  AND d.ledger_code = ?
  AND d.trans_date <= ?
$DATERANGE
 GROUP BY account_num, a.name, code, account_type, account_type_num
 UNION ALL
 SELECT
  2 AS level,
  'TOTAL' AS label,
  NULL,
  NULL,
  NULL,
  NULL,
  SUM(
    CASE WHEN d.dorc = true THEN
      d.amount
    ELSE 0.00
    END
  ) AS debit,
  SUM(
    CASE WHEN d.dorc = false THEN
      d.amount
    ELSE 0.00
    END
  ) AS credit
 FROM debitcredit_tb AS d
 WHERE
  d.ledger_code = ?
  AND d.trans_date <= ?
$DATERANGE
 ORDER BY level, label
ENDOF
  $self->{DATA} .= $self->dbHandle->readHTML(sql => $self->{LIST}[0], plug => \@plug, displayNULLAs => '', headers => $self->{headers}, displayNumRows => 0, tableClass => 'trial');
  if ($self->{dbHandle}->error) { $self->error($self->{dbHandle}->errorMessage); return 0; }
  return 1;
}

sub printReport
{
  my $self = shift;
  if (!defined $self->{DATA})
  {
    $self->select(@_);
    if ($self->error) { return undef; }
  }
  my %f;
  $f{body} .= $self->{DATA} . $self->{template}->{report};
  $f{javascript} .= $self->{javascript}->{report};
  $f{css} .= $self->{css}->{report};
  return %f;
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal::Accounting::Objects::Base(3)

=cut
